#!/usr/bin/env python
"""
Unittests for Utilities functions
"""

from builtins import object
import unittest

from Utils.Utilities import makeList, makeNonEmptyList, strToBool, \
    safeStr, rootUrlJoin, zipEncodeStr, lowerCmsHeaders, getSize, \
    encodeUnicodeToBytes, diskUse, numberCouchProcess, normalize_spaces


class UtilitiesTests(unittest.TestCase):
    """
    unittest for Utilities functions
    """

    def testMakeList(self):
        """
        Test the makeList function
        """
        self.assertEqual(makeList([]), [])
        self.assertEqual(makeList(""), [])
        self.assertEqual(makeList(['123']), ['123'])
        self.assertEqual(makeList([456]), [456])
        self.assertListEqual(makeList(['123', 456, '789']), ['123', 456, '789'])

        self.assertEqual(makeList('123'), ['123'])
        self.assertEqual(makeList('123'), ['123'])
        self.assertListEqual(makeList('123,456'), ['123', '456'])
        self.assertListEqual(makeList('123,456'), ['123', '456'])
        self.assertListEqual(makeList('["aa","bb","cc"]'), ['aa', 'bb', 'cc'])
        self.assertListEqual(makeList(' ["aa", "bb", "cc"] '), ['aa', 'bb', 'cc'])

        self.assertRaises(ValueError, makeList, 123)
        self.assertRaises(ValueError, makeList, 123.456)
        self.assertRaises(ValueError, makeList, {1: 123})

    def testLowerCmsHeaders(self):
        "Test lowerCmsHeaders function"
        val = 'cms-xx-yy'
        headers = {'CAPITAL': 1, 'Camel': 1, 'Cms-Xx-Yy': val, 'CMS-XX-YY': val, 'cms-xx-yy': val}
        lheaders = lowerCmsHeaders(headers)
        self.assertEqual(sorted(lheaders.keys()), sorted(['CAPITAL', 'Camel', val]))
        self.assertEqual(lheaders['CAPITAL'], 1)
        self.assertEqual(lheaders['Camel'], 1)
        self.assertEqual(lheaders[val], val)
        self.assertEqual(len(lheaders), 3)

    def testMakeNonEmptyList(self):
        """
        Test the makeNonEmptyList function.
        It has exactly the same behaviour as makeList, but throws an exception
        for empty list or string.
        """
        self.assertEqual(makeList(['123']), makeNonEmptyList(['123']))
        self.assertListEqual(makeList(['123', 456, '789']), makeNonEmptyList(['123', 456, '789']))

        self.assertListEqual(makeList('123,456'), makeNonEmptyList('123, 456'))
        self.assertListEqual(makeList('["aa","bb","cc"]'), makeNonEmptyList('["aa", "bb", "cc"]'))

        self.assertRaises(ValueError, makeNonEmptyList, [])
        self.assertRaises(ValueError, makeNonEmptyList, "")

    def testStrToBool(self):
        """
        Test the strToBool function.
        """
        for v in [True, "True", "TRUE", "true"]:
            self.assertTrue(strToBool(v))
        for v in [False, "False", "FALSE", "false"]:
            self.assertFalse(strToBool(v))

        for v in [1, 0, "", "alan", [], [''], {'a': 123}]:
            self.assertRaises(ValueError, strToBool, v)

    def testSafeStr(self):
        """
        Test the safeStr function.
        """
        for v in ['123', '123', 123]:
            self.assertEqual(safeStr(v), '123')
        self.assertEqual(safeStr(123.45), '123.45')
        self.assertEqual(safeStr(False), 'False')
        self.assertEqual(safeStr(None), 'None')
        self.assertEqual(safeStr(""), "")

        for v in [[1, 2], {'x': 123}, set([1])]:
            self.assertRaises(ValueError, safeStr, v)

    def testRootUrlJoin(self):
        """
        Test the testRootUrlJoin function.
        """
        urlbase = "root://site.domain//somepath"
        self.assertEqual(rootUrlJoin(urlbase, "extend"), "root://site.domain//somepath/extend")
        urlbase = "random"
        self.assertEqual(rootUrlJoin(urlbase, "extend"), None)
        urlbase = None
        self.assertEqual(rootUrlJoin(urlbase, "extend"), None)

    def testZipEncodeStr(self):
        """
        Test the zipEncodeStr function.
        """
        message = """
%MSG-s CMSException:  AfterFile 02-Jun-2010 14:31:43 CEST PostEndRun
cms::Exception caught in cmsRun
---- EventProcessorFailure BEGIN
EventProcessingStopped
---- ScheduleExecutionFailure BEGIN
ProcessingStopped
---- InvalidReference BEGIN
BadRefCore Attempt to dereference a RefCore containing an invalid
ProductID has been detected. Please modify the calling
code to test validity before dereferencing.
cms::Exception going through module PatMCMatching/analyzePatMCMatching run: 1 lumi: 666672 event: 305
---- InvalidReference END
Exception going through path p
---- ScheduleExecutionFailure END
an exception occurred during current event processing
cms::Exception caught in CMS.EventProcessor and rethrown
---- EventProcessorFailure END
"""
        encodedMessage = \
            'eNp1j8FqwzAMhu95Cl0G2yEhaXvyrU3dkkFHqfcCnq02hkQOtlz6+HM2MrbDdBLS9/1CxdNJHcsI7UnJh8GJnScBsL0yhoMbEOpV+ZqoXNVNDc1GrBuxWUMr1TucfWRJ9pKoMGMU4scHo9OtZ3C5G+O8L3OBvCPxOXiDMfpw0G5IAWEnj91b8Xvn6KbYTxPab0+ZHm0aUD7QpDn/r/qP1dFdD85e8IoBySz0Ts+j1md9y4zjxMAebGYWTsMCGE+sHeVk0JS/+Qqc79lkuNtDryN8IBLAc1VVL5+o0W8i'
        encodedMessage = encodeUnicodeToBytes(encodedMessage)
        self.assertEqual(zipEncodeStr(message, maxLen=300, compressLevel=9, steps=10, truncateIndicator=" (...)"),
                         encodedMessage)
        # Test different maximum lengths
        # Encoded message should always be less than the maximum limit.
        for maxLen in (800, 500, 20):
            self.assertLessEqual(
                    len(zipEncodeStr(message, maxLen=maxLen, compressLevel=9, steps=10, truncateIndicator=" (...)")),
                    maxLen)

    def testGetSize(self):
        """
        Test the getSize function.
        """
        for item in (1234, 1234.1234, set([12, 34]), "test", (1, 2), {'k1': 'v1'}):
            self.assertTrue(getSize(item) > 20)
        with self.assertRaises(TypeError):
            getSize(zipEncodeStr)

        class TestClass1():
            def __init__(self):
                self.data = "blah"

        cls1 = TestClass1()
        print(getSize(cls1)) # py2: 1129, py3: 205
        self.assertTrue(getSize(cls1) > 200)

        class TestClass2(object):
            """
            In python2, classes that inherit from `object` have a smaller
            memory footprint
            """
            def __init__(self):
                self.data = "blah"

        cls2 = TestClass2()
        print(getSize(cls2)) # py2: 426, py3: 205
        self.assertTrue(getSize(cls2) > 200)

    def testDiskUse(self):
        """
        Test the `diskUse` function.
        """
        data = diskUse()

        # `data` is a list of dictionaries with three keys each:
        # [
        #   {
        #    "filesystem": "devtmpfs",
        #    "mounted": "/dev",
        #    "percent": "0%"
        #  },
        #  {
        #    "filesystem": "/dev/vda1",
        #    "mounted": "/",
        #    "percent": "76%"
        #  }
        #  [...]
        # ]
        print(data)

        # assuming nodes will always have at least 3 partitions/mount points
        self.assertTrue(len(data) > 2)

        # - 1. every dictionary in the list needs to contain all of the 
        #   expexted keys and their values must be a non-empty string
        # - 2. the value of the "percent" key needs to be either "100" 
        #   or a one or two digits int, followed by a percentage character "%"
        # - 3. there must be at least one entry whose filesystem contains `/dev/*`
        filesystem_is_dev = 0
        for entry in data:
            print(entry)

            # check 1.
            self.assertTrue("filesystem" in entry)
            self.assertTrue("mounted" in entry)
            self.assertTrue("percent" in entry)
            for value in entry.values():
                self.assertTrue(isinstance(value, str))
                self.assertTrue(len(value) > 0)

            import re
            # ref: https://docs.python.org/3/library/re.html#re.search
            # re.search() returns None if no position in the string matches the pattern

            # check 2.
            self.assertTrue(re.search("^(100|[0-9]{1,2})%$", entry["percent"]) is not None)

            # check 3.
            if re.search("^/dev/.+$" , entry["filesystem"]):
                filesystem_is_dev += 1

        self.assertTrue(filesystem_is_dev > 0)

    def testNormalizeSpaces(self):
        """
        Test the `normalize_spaces` function.
        """
        data = normalize_spaces("bla bla     bla    ")
        expect = "bla bla bla"
        self.assertEqual(data, expect)

    def testNumberCouchProcess(self):
        """
        Test the `numberCouchProcess` function.
        """
        data = numberCouchProcess()
        # there should be at least one process, but who knows...
        self.assertTrue(data >= 0)


if __name__ == '__main__':
    unittest.main()



from builtins import range
import time
import random

NUM_OF_REQUEST = 1
ITERATIONS =1
NUM_OF_JOBS_PER_REQUEST = 11

def generate_reqmgr_schema(number=NUM_OF_REQUEST):
    """
    generate the request with following structure
    doc["_id"] = schema['RequestName']
    doc["workflow"] = schema['RequestName']
    doc["requestor"] = schema['Requestor']
    doc["campaign"] = schema['Campaign']
    doc["request_type"] = schema['RequestType']
    doc["priority"] = schema['RequestPriority']
    doc["group"] = schema['Group']
    doc["request_date"] = schema['RequestDate']
    doc["type"] = "reqmgr_request"
    # additional field
    doc["inputdataset"] = schema.get('InputDataset', "")
    # additional field for Analysis work
    doc["vo_group"] = schema.get('VoGroup', "")
    doc["vo_role"] = schema.get('VoRole', "")
    doc["user_dn"] = schema.get('RequestorDN', "")
    doc["async_dest"] = schema.get('asyncDest', "")
    doc["dbs_url"] = schema.get("DbsUrl", "")
    doc["publish_dbs_url"] = schema.get("PublishDbsUrl", "")
    # team name is not yet available need to be updated in assign status
    #doc['team'] = schema['team']
    """
    docs = []
    for i in range(number):
        doc = {"RequestName": "test_workflow_%s" % i,
               "InputDataset": "/Photon/Run2011A-v1/RAW",
               "Group": "cmsdataops",
               "RequestDate": [2012, 1, 11, 17, 49, 50],
               "Campaign": "SryuTest-%s" % (i % 5),
               "RequestPriority": "1",
               "Requestor": "cmsdataops",
               "RequestType": "ReReco"
               #"site_white_list": ["T1_DE_KIT"],
               #"team": "cmsdataops"
                }
        docs.append(doc)
    return docs


def generate_reqmgr_requests(number=NUM_OF_REQUEST):
    """
    generate the request with following structure
      {
       "_id": "cmsdataops_sryu_test4_120111_114950_128",
       "_rev": "6-02b17b4eabcf333e7499c0fa0ae5055b",
       "inputdataset": "/Photon/Run2011A-v1/RAW",
       "group": "cmsdataops",
       "request_date": [2012, 1, 11, 17, 49, 50],
       "campaign": "SryuTest2",
       "workflow": "cmsdataops_sryu_test4_120111_114950_128",
       "priority": "1",
       "requestor": "cmsdataops",
       "request_type": "ReReco",
       "type": "reqmgr_request",
       "request_status": [
           {
               "status": "new",
               "update_time": 1326304190
           },
           {
               "status": "assignment-approved",
               "update_time": 1326304216
           },
           {
               "status": "assigned",
               "update_time": 1326304227
           }
       ],
       "site_white_list": [
           "T1_DE_KIT"
       ],
       "team": "cmsdataops"
    }
    """
    docs = []
    for i in range(number):
        doc = {"_id": "test_workflow_%s" % i,
               "inputdataset": "/Photon/Run2011A-v1/RAW",
               "group": "cmsdataops",
               "request_date": [2012, 1, 11, 17, 49, 50],
               "campaign": "SryuTest-%s" % (i % 5),
               "workflow": "test_workflow_%s" % i,
               "priority": "1",
               "requestor": "cmsdataops",
               "request_type": "ReReco",
               "type": "reqmgr_request",
               "request_status": [
                                  {"status": "new", "update_time": 1326304190},
                                  {"status": "assignment-approved", "update_time": 1326304216},
                                  {"status": "assigned", "update_time": 1326304227}
                                 ],
                "site_white_list": ["T1_DE_KIT"],
                "team": "cmsdataops"
                }
        docs.append(doc)
    return docs


def generate_agent_requests(number=NUM_OF_REQUEST, iterations=ITERATIONS):
    """
    generate the request with following structure
      {
       "_id": "af27057919546ff8f3fc8d7f18233355",
       "_rev": "1-181021c38a5444676d7718f42ffa9a89",
       "status": {
           "inWMBS": 1,
           "queued": {
               "retry": 1
           }
       },
       "workflow": "cmsdataops_sryu_test4_120111_114950_128",
       "timestamp": 1326306397,
       "sites": {
           "T1_DE_KIT": {
               "queued": {
                   "retry": 1
               }
           }
       },
       "agent": "WMAgent",
       "team": "team1,team2,cmsdataops",
       "agent_url": "cms-xen39.fnal.gov",
       "type": "agent_request"
       }
    """
    current_time = int(time.time())
    docs = []
    for cycle in range(iterations):
        for i in range(number):
            doc = {"status": {"inWMBS": 12,
                              "submitted": {"retry": 2, "running": 2, "pending": 2, "first": 2},
                              "failure": {"exception": 2, "create": 2, "submit": 2},
                              "queued": {"retry": 2, "first": 2},
                              "canceled": 2,
                              "cooloff": 2,
                              "success": 2
                             },

                "workflow": "test_workflow_%s" % i,
                "timestamp": current_time + (cycle * 10),
                "sites": {"T1_DE_KIT":
                             {
                              "submitted": {"retry": 1, "running": 1, "pending": 1, "first": 1},
                              "failure": {"exception": 1, "create": 1, "submit": 1},
                              "queued": {"retry": 1, "first": 1},
                              "canceled": 1,
                              "cooloff": 1,
                              "success": 1
                             },
                          "T1_US_FNAL":
                             {
                              "submitted": {"retry": 1, "running": 1, "pending": 1, "first": 1},
                              "failure": {"exception": 1, "create": 1, "submit": 1},
                              "queued": {"retry": 1, "first": 1},
                              "canceled": 1,
                              "cooloff": 1,
                              "success": 1
                             }
                          },
                "agent": "WMAgent",
                "agent_teams": "cmsdataops",
                "agent_url": "cms-xen39.fnal.gov",
                "type": "agent_request"
            }
            docs.append(doc)
    return docs


def generate_jobsummary(request, number=NUM_OF_JOBS_PER_REQUEST):
    """
    jobSummary = {"_id": "jobid_1",  //jobid
                  "type": "jobsummary", // setvalue
                  "retrycount": job["retry_count"],
                  "workflow": workflow1, //request name
                  "task": job["task"],
                  "state": success,
                  "site": T1_US_FNAL,
                  "exitcode": 123,
                  "errors": errmsgs,
                  "lumis": inputs,
                  "output": outputs }

    errmsgs = {}
    inputs = []
    for step in fwjrDocument["fwjr"]["steps"]:
        if "errors" in fwjrDocument["fwjr"]["steps"][step]:
            errmsgs[step] = [error for error in fwjrDocument["fwjr"]["steps"][step]["errors"]]
        if "input" in fwjrDocument["fwjr"]["steps"][step] and "source" in fwjrDocument["fwjr"]["steps"][step]["input"]:
            inputs.extend( [source["runs"] for source in fwjrDocument["fwjr"]['steps'][step]["input"]["source"] if "runs" in source] )
    outputs = [ {'type': singlefile.get('module_label', None),
                 'lfn': singlefile.get('lfn', None),
                 'location': singlefile.get('locations', None),
                 'checksums': singlefile.get('checksums', {}),
                     'size': singlefile.get('size', None) } for singlefile in job["fwjr"].getAllFiles() if singlefile ]


    job status
    ['new', 'created', 'executing', 'complete', 'createfailed', 'submitfailed',
     'jobfailed', 'createcooloff',  'submitcooloff', 'jobcooloff', 'success',
     'exhausted', 'killed']
    """

    #TODO: Make more realistic
    docs = []
    statusList = ['new', 'created', 'executing', 'complete', 'createfailed', 'submitfailed',
     'jobfailed', 'createcooloff',  'submitcooloff', 'jobcooloff', 'success',
     'exhausted', 'killed']

    for i in range(number):
        status = statusList[random.randint(0, len(statusList)-1)]
        errmsgs = {}
        if status.find("failed"):
            exitCode = 666
            errmsgs["step1"] = {}
            errmsgs["step1"]["out"] = {}
            errmsgs["step1"]["out"]["type"] = "test error"
        else:
            exitCode = 0

        jobSummary = {"_id": "jobid_%s_%s" % (request, i),
                  "type": "jobsummary",
                  "retrycount": random.randint(0,5),
                  "workflow": request,
                  "task": "/%s/task_%s" % (request, i),
                  "state": status,
                  "site": "T1_US_FNAL",
                  "exitcode": exitCode,
                  "errors": errmsgs,
                  "lumis": [[123, 124], [567, 879]],
                  "output": [ {'type': "test-type",
                               'lfn': "/somewhere/file.root",
                               'location': ['T1_US_FNAL'],
                               'checksums': {'adler32': 'abc123', 'cksum': 'cdf123'},
                               'size': "1000" }  ]
            }
        docs.append(jobSummary)
    return docs

sample_request_info = {'AcquisitionEra': 'Integ_Test',
 'AgentJobInfo': {'vocms008.cern.ch:9999': {'_id': '1444db7834fdedaf68fbf7498330adab',
                                            '_rev': '1-82d7cef10f3df98b481609152b048dd5',
                                            'agent': 'WMAgent',
                                            'agent_team': 'testbed-dev',
                                            'agent_url': 'vocms008.cern.ch:9999',
                                            'sites': {'T1_US_FNAL': {'submitted': {'first': 35,
                                                                                   'running': 43}},
                                                      'T2_CH_CERN': {'submitted': {'first': 19,
                                                                                   'running': 19}},
                                                      'T2_CH_CERN_HLT': {'submitted': {'first': 46,
                                                                                       'pending': 34,
                                                                                       'running': 4}}},
                                            'status': {'inWMBS': 100,
                                                       'submitted': {'first': 100,
                                                                     'pending': 34,
                                                                     'running': 66}},
                                            'tasks': {'/sryu_MonteCarloFromGEN_wq_testt_160220_214518_7489/MonteCarloFromGEN': {'jobtype': 'Production',
                                                                                                                                'sites': {'T1_US_FNAL': {'submitted': {'first': 35,
                                                                                                                                                                       'running': 43}},
                                                                                                                                          'T2_CH_CERN': {'submitted': {'first': 19,
                                                                                                                                                                       'running': 19}},
                                                                                                                                          'T2_CH_CERN_HLT': {'submitted': {'first': 46,
                                                                                                                                                                           'pending': 34,
                                                                                                                                                                           'running': 4}}},
                                                                                                                                'status': {'submitted': {'first': 100,
                                                                                                                                                         'pending': 34,
                                                                                                                                                         'running': 66}},
                                                                                                                                'subscription_status': {'finished': 0,
                                                                                                                                                        'open': 1,
                                                                                                                                                        'total': 1,
                                                                                                                                                        'updated': 1456032349}},
                                                      '/sryu_MonteCarloFromGEN_wq_testt_160220_214518_7489/MonteCarloFromGEN/LogCollect': {'jobtype': 'LogCollect',
                                                                                                                                           'subscription_status': {'finished': 0,
                                                                                                                                                                   'open': 1,
                                                                                                                                                                   'total': 1,
                                                                                                                                                                   'updated': 1456032349}},
                                                      '/sryu_MonteCarloFromGEN_wq_testt_160220_214518_7489/MonteCarloFromGEN/MonteCarloFromGENCleanupUnmergedRAWSIMoutput': {'jobtype': 'Cleanup',
                                                                                                                                                                             'subscription_status': {'finished': 0,
                                                                                                                                                                                                     'open': 1,
                                                                                                                                                                                                     'total': 1,
                                                                                                                                                                                                     'updated': 1456032349}},
                                                      '/sryu_MonteCarloFromGEN_wq_testt_160220_214518_7489/MonteCarloFromGEN/MonteCarloFromGENMergeRAWSIMoutput': {'jobtype': 'Merge',
                                                                                                                                                                   'subscription_status': {'finished': 0,
                                                                                                                                                                                           'open': 1,
                                                                                                                                                                                           'total': 1,
                                                                                                                                                                                           'updated': 1456032349}},
                                                      '/sryu_MonteCarloFromGEN_wq_testt_160220_214518_7489/MonteCarloFromGEN/MonteCarloFromGENMergeRAWSIMoutput/MonteCarloFromGENRAWSIMoutputMergeLogCollect': {'jobtype': 'LogCollect',
                                                                                                                                                                                                                'subscription_status': {'finished': 0,
                                                                                                                                                                                                                                        'open': 1,
                                                                                                                                                                                                                                        'total': 1,
                                                                                                                                                                                                                                        'updated': 1456032349}}},
                                            'timestamp': 1456033921,
                                            'type': 'agent_request',
                                            'workflow': 'sryu_MonteCarloFromGEN_wq_testt_160220_214518_7489'}},
 'AutoApproveSubscriptionSites': [],
 'BlockBlacklist': [],
 'BlockCloseMaxEvents': 20000000,
 'BlockCloseMaxFiles': 500,
 'BlockCloseMaxSize': 5000000000000,
 'BlockCloseMaxWaitTime': 14400,
 'BlockWhitelist': [],
 'CMSSWVersion': 'CMSSW_5_3_19',
 'Campaign': 'reqmgr2-validation',
 'Comments': 'MCFromGEN LumiBased splitting with 1l per job. Half an hour opened',
 'ConfigCacheID': '1ad063a0d73c1d81143b4182cbf84793',
 'ConfigCacheUrl': 'https://cmsweb.cern.ch/couchdb',
 'CouchDBName': 'reqmgr_config_cache',
 'CouchURL': 'https://reqmgr2-dev.cern.ch/couchdb',
 'CouchWorkloadDBName': 'reqmgr_workload_cache',
 'CustodialSites': [],
 'CustodialSubType': 'Move',
 'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
 'Dashboard': 'integration',
 'DbsUrl': 'https://cmsweb-testbed.cern.ch/dbs/int/global/DBSReader',
 'FilterEfficiency': 1,
 'GlobalTag': 'START53_V7C::All',
 'GracePeriod': 300,
 'Group': 'DATAOPS',
 'HardTimeout': 129900,
 'InputDataset': '/QDTojWinc_NC_M-1200_TuneZ2star_8TeV-madgraph/Summer12pLHE-DMWM_Validation_DONOTDELETE_Alan_TEST-v1/GEN',
 'LumisPerJob': 1,
 'MaxMergeEvents': 50000,
 'MaxMergeSize': 4294967296,
 'Memory': 2300,
 'MergedLFNBase': '/store/backfill/1',
 'MinMergeSize': 2147483648,
 'NonCustodialSites': [],
 'OpenRunningTimeout': 1800,
 'OutputDatasets': ['/QDTojWinc_NC_M-1200_TuneZ2star_8TeV-madgraph/Summer12-START53_V7C-v2/GEN-SIM'],
 'PrepID': 'B2G-Summer12-00736',
 'PrimaryDataset': 'QDTojWinc_NC_M-1200_TuneZ2star_8TeV-madgraph',
 'ProcessingString': 'MonteCarloFromGEN_reqmgr2-test',
 'ProcessingVersion': 1,
 'RequestDate': [2016, 2, 20, 20, 45, 18],
 'RequestName': 'sryu_MonteCarloFromGEN_wq_testt_160220_214518_7489',
 'RequestNumEvents': 50000,
 'RequestPriority': 90000,
 'RequestStatus': 'running-closed',
 'RequestString': 'MonteCarloFromGEN_wq_testt',
 'RequestTransition': [{'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'new',
                        'UpdateTime': 1456001118},
                       {'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'assignment-approved',
                        'UpdateTime': 1456001121},
                       {'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'assigned',
                        'UpdateTime': 1456001121},
                       {'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'acquired',
                        'UpdateTime': 1456002499},
                       {'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'running-open',
                        'UpdateTime': 1456033504},
                       {'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'running-closed',
                        'UpdateTime': 1456033517}],
 'RequestType': 'MonteCarloFromGEN',
 'RequestWorkflow': 'https://reqmgr2-dev.cern.ch/couchdb/reqmgr_workload_cache/sryu_MonteCarloFromGEN_wq_testt_160220_214518_7489/spec',
 'Requestor': 'sryu',
 'RequestorDN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
 'RunBlacklist': [],
 'RunWhitelist': [],
 'ScramArch': 'slc6_amd64_gcc472',
 'SiteBlacklist': [],
 'SiteWhitelist': ['T1_US_FNAL', 'T2_CH_CERN'],
 'SizePerEvent': 1154,
 'SoftTimeout': 129600,
 'SplittingAlgo': 'LumiBased',
 'SubscriptionPriority': 'Low',
 'Teams': ['testbed-dev'],
 'TimePerEvent': 16.87,
 'TotalEstimatedJobs': 100,
 'TotalInputEvents': 2500,
 'TotalInputFiles': 1,
 'TotalInputLumis': 100,
 'TotalTime': 28800,
 'UnmergedLFNBase': '/store/unmerged',
 '_id': 'sryu_MonteCarloFromGEN_wq_testt_160220_214518_7489'}

sample_complete = {'AcquisitionEra': 'Integ_Test',
 'AgentJobInfo': {'vocms008.cern.ch:9999': {'_id': '1444db7834fdedaf68fbf7498394376c',
                                            '_rev': '1-0250fbe480c56e57b2c939ff491aab8b',
                                            'agent': 'WMAgent',
                                            'agent_team': 'testbed-dev',
                                            'agent_url': 'vocms008.cern.ch:9999',
                                            'sites': {'T2_CH_CERN_HLT': {'failure': {'exception': 1},
                                                                         'success': 1}},
                                            'status': {'failure': {'exception': 1},
                                                       'success': 1},
                                            'tasks': {'/sryu_DQMHarvesting_wq_testt_160220_214501_5235/EndOfRunDQMHarvest': {'sites': {'T2_CH_CERN_HLT': {'failure': {'exception': 1}}},
                                                                                                                             'status': {'failure': {'exception': 1}}},
                                                      '/sryu_DQMHarvesting_wq_testt_160220_214501_5235/EndOfRunDQMHarvest/EndOfRunDQMHarvestLogCollect': {'sites': {'T2_CH_CERN': {'cmsRunCPUPerformance': {'totalEventCPU': 0,
                                                                                                                                                                                                            'totalJobCPU': 0,
                                                                                                                                                                                                            'totalJobTime': 0},
                                                                                                                                                                                   'dataset': {},
                                                                                                                                                                                   'inputEvents': 0,
                                                                                                                                                                                   'wrappedTotalJobTime': 18},
                                                                                                                                                                    'T2_CH_CERN_HLT': {'success': 1}},
                                                                                                                                                          'status': {'success': 1}}},
                                            'timestamp': 1456037523,
                                            'type': 'agent_request',
                                            'workflow': 'sryu_DQMHarvesting_wq_testt_160220_214501_5235'}},
 'AutoApproveSubscriptionSites': [],
 'BlockBlacklist': [],
 'BlockCloseMaxEvents': 250000000,
 'BlockCloseMaxFiles': 500,
 'BlockCloseMaxSize': 5000000000000,
 'BlockCloseMaxWaitTime': 72000,
 'CMSSWVersion': 'CMSSW_7_3_1_patch1',
 'Campaign': 'reqmgr2-validation',
 'Comments': 'DQMHarvest spec with multiRun DQMHarvestUnit',
 'ConfigCacheUrl': 'https://cmsweb.cern.ch/couchdb',
 'CouchDBName': 'reqmgr_config_cache',
 'CouchURL': 'https://reqmgr2-dev.cern.ch/couchdb',
 'CouchWorkloadDBName': 'reqmgr_workload_cache',
 'CustodialSites': [],
 'CustodialSubType': 'Move',
 'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
 'DQMConfigCacheID': 'e554f238a2ee8bb969248a343f6195d1',
 'DQMHarvestUnit': 'multiRun',
 'DQMUploadUrl': 'https://cmsweb-testbed.cern.ch/dqm/dev',
 'Dashboard': 'integration',
 'DbsUrl': 'https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader',
 'GlobalTag': 'GR_R_73_V0A',
 'GracePeriod': 1000,
 'Group': 'DATAOPS',
 'HardTimeout': 130600,
 'InputDataset': '/MinimumBias/CMSSW_7_3_1_patch1-GR_R_73_V0A_RelVal_run2010A-v1/DQMIO',
 'MaxMergeEvents': 100000,
 'MaxMergeSize': 4294967296,
 'Memory': 2200,
 'MergedLFNBase': '/store/relval',
 'MinMergeSize': 2147483648,
 'NonCustodialSites': [],
 'OutputDatasets': [],
 'ProcessingString': 'DQMHarvesting_reqmgr2-test',
 'ProcessingVersion': 1,
 'RequestDate': [2016, 2, 20, 20, 45, 1],
 'RequestName': 'sryu_DQMHarvesting_wq_testt_160220_214501_5235',
 'RequestPriority': 140000,
 'RequestStatus': 'completed',
 'RequestString': 'DQMHarvesting_wq_testt',
 'RequestTransition': [{'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'new',
                        'UpdateTime': 1456001101},
                       {'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'assignment-approved',
                        'UpdateTime': 1456001106},
                       {'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'assigned',
                        'UpdateTime': 1456001106},
                       {'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'acquired',
                        'UpdateTime': 1456002496},
                       {'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'running-open',
                        'UpdateTime': 1456033516},
                       {'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'running-closed',
                        'UpdateTime': 1456033519},
                       {'DN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
                        'Status': 'completed',
                        'UpdateTime': 1456035903}],
 'RequestType': 'DQMHarvest',
 'RequestWorkflow': 'https://reqmgr2-dev.cern.ch/couchdb/reqmgr_workload_cache/sryu_DQMHarvesting_wq_testt_160220_214501_5235/spec',
 'Requestor': 'sryu',
 'RequestorDN': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=sryu/CN=676041/CN=Seang Chan Ryu',
 'RunWhitelist': [138937,
                  138934,
                  138924,
                  138923,
                  139790,
                  139789,
                  139788,
                  139787,
                  144086,
                  144085,
                  144084,
                  144083,
                  144011],
 'ScramArch': 'slc6_amd64_gcc491',
 'SiteBlacklist': [],
 'SiteWhitelist': ['T1_US_FNAL', 'T2_CH_CERN'],
 'SizePerEvent': 1234,
 'SoftTimeout': 129600,
 'SubscriptionPriority': 'Low',
 'Teams': ['testbed-dev'],
 'TimePerEvent': 2,
 'TotalEstimatedJobs': 1,
 'TotalInputEvents': 0,
 'TotalInputFiles': 2,
 'TotalInputLumis': 21,
 'UnmergedLFNBase': '/store/unmerged',
 '_id': 'sryu_DQMHarvesting_wq_testt_160220_214501_5235',
 'mergedLFNBase': '/store/relval',
 'unmergedLFNBase': '/store/unmerged'}



#!/usr/bin/env python
"""
_Tier1ReRecoWorkload_



"""
from WMCore.WMSpec.WMWorkload import newWorkload

DBSURL = "https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader"

#  //
# // Set up the basic workload task and step structure
#//
workload = newWorkload("MultiTaskReReco")
workload.setOwner("WMTest")
workload.setStartPolicy('DatasetBlock')
workload.setEndPolicy('SingleShot')

#  //
# // set up the production task
#//
rereco = workload.newTask("ReReco1")
rerecoCmssw = rereco.makeStep("cmsRun1")
rerecoCmssw.setStepType("CMSSW")
skimStageOut = rerecoCmssw.addStep("stageOut1")
skimStageOut.setStepType("StageOut")
skimLogArch = rerecoCmssw.addStep("logArch1")
skimLogArch.setStepType("LogArchive")
rereco.applyTemplates()
rereco.setSplittingAlgorithm("FileBased", files_per_job = 1)
rereco.addInputDataset(
    name="/Cosmics/ComissioningHI-PromptReco-v1/RECO",
    primary = "Cosmics",
    processed = "ComissioningHI-PromptReco-v1",
    tier = "RECO",
    dbsurl = DBSURL)

#  //
# // rereco cmssw step
#//
#
# TODO: Anywhere helper.data is accessed means we need a method added to the
# type based helper class to provide a clear API.
rerecoCmsswHelper = rerecoCmssw.getTypeHelper()


rerecoCmsswHelper.cmsswSetup(
    "CMSSW_3_1_2",
    softwareEnvironment = " . /uscmst1/prod/sw/cms/bashrc prod"
    )

rerecoCmsswHelper.setDataProcessingConfig(
    "cosmics", "promptReco", globalTag = "GLOBAL::BALLS",
    writeTiers = ['RECO'])

rerecoCmsswHelper.addOutputModule(
    "outputRECO", primaryDataset = "Primary",
    processedDataset = "Processed",
    dataTier = "RECO")

pileupConfig = {"data" : ["/HighPileUp/Run2011A-v1/RAW"]}
rerecoCmsswHelper.setupPileup(pileupConfig, DBSURL)

#Add a stageOut step
skimStageOutHelper = skimStageOut.getTypeHelper()
skimLogArchHelper  = skimLogArch.getTypeHelper()


rereco.addGenerator("BasicNaming")

#  //
# // set up the production task
#//
rereco = workload.newTask("ReReco2")
rerecoCmssw = rereco.makeStep("cmsRun1")
rerecoCmssw.setStepType("CMSSW")
skimStageOut = rerecoCmssw.addStep("stageOut1")
skimStageOut.setStepType("StageOut")
skimLogArch = rerecoCmssw.addStep("logArch1")
skimLogArch.setStepType("LogArchive")
rereco.applyTemplates()
rereco.setSplittingAlgorithm("FileBased", files_per_job = 1)
rereco.addInputDataset(
    name="/Cosmics/ComissioningHI-PromptReco-v1/RECO",
    primary = "Cosmics",
    processed = "ComissioningHI-PromptReco-v1",
    tier = "RECO",
    dbsurl = DBSURL)
#  //
# // rereco cmssw step
#//
#
# TODO: Anywhere helper.data is accessed means we need a method added to the
# type based helper class to provide a clear API.
rerecoCmsswHelper = rerecoCmssw.getTypeHelper()


rerecoCmsswHelper.cmsswSetup(
    "CMSSW_3_1_2",
    softwareEnvironment = " . /uscmst1/prod/sw/cms/bashrc prod"
    )

rerecoCmsswHelper.setDataProcessingConfig(
    "cosmics", "promptReco", globalTag = "GLOBAL::BALLS",
    writeTiers = ['RECO'])

rerecoCmsswHelper.addOutputModule(
    "outputRECO", primaryDataset = "Primary",
    processedDataset = "Processed",
    dataTier = "RECO")

#Add a stageOut step
skimStageOutHelper = skimStageOut.getTypeHelper()
skimLogArchHelper  = skimLogArch.getTypeHelper()


rereco.addGenerator("BasicNaming")

workload.setSiteWhitelist(["T2_XX_SiteA", "T2_XX_SiteB", "T2_XX_SiteC"])



#!/usr/bin/env python
"""
Helper class for RequestManager interaction
"""
from builtins import object
from future.utils import viewvalues

import logging
import os
import socket
import threading
import traceback
from operator import itemgetter

from WMCore import Lexicon
from WMCore.Database.CMSCouch import CouchError, CouchNotFoundError
from WMCore.Database.CouchUtils import CouchConnectionError
from WMCore.ReqMgr.DataStructs.RequestStatus import REQUEST_STATE_LIST
from WMCore.Services.LogDB.LogDB import LogDB
from WMCore.Services.ReqMgr.ReqMgr import ReqMgr
from WMCore.Services.Rucio.Rucio import Rucio
from WMCore.WorkQueue.WorkQueueExceptions import WorkQueueWMSpecError, WorkQueueNoWorkError, TERMINAL_EXCEPTIONS


class WorkQueueReqMgrInterface(object):
    """Helper class for ReqMgr interaction"""

    def __init__(self, **kwargs):
        if not kwargs.get('logger'):
            kwargs['logger'] = logging
        self.logger = kwargs['logger']
        self.rucio = Rucio(kwargs.get("rucioAccount", "wmcore_transferor"),
                           configDict=dict(logger=self.logger))
        # this will break all in one test
        self.reqMgr2 = ReqMgr(kwargs.get("reqmgr2_endpoint", None))

        centralurl = kwargs.get("central_logdb_url", "")
        identifier = kwargs.get("log_reporter", "")

        # set the thread name before creat the log db.
        # only sets that when it is not set already
        myThread = threading.currentThread()
        if myThread.getName() == "MainThread":
            myThread.setName(self.__class__.__name__)

        self.logdb = LogDB(centralurl, identifier, logger=self.logger)

    def __call__(self, queue):
        """Synchronize WorkQueue and RequestManager"""
        # ensure log records go to the correct logger object
        queue.logger = self.logger
        msg = ''
        try:  # Fetch data from ReqMgr and propagate work cancellation to workqueue
            self.logger.info("Canceling aborted and force-completed requests")
            count = self.cancelWork(queue)
            msg += "Work canceled: %s, " % count
        except Exception as ex:
            self.logger.exception("Generic error while canceling work. Details: %s", str(ex))

        try:  # Close requests that no longer need to be open for new data
            self.logger.info("Closing open requests")
            workClosed = queue.closeWork()
            msg += "Work closed: %d, " % len(workClosed)
        except Exception as ex:
            errorMsg = "Generic error while closing open requests. Details: %s"
            self.logger.exception(errorMsg, str(ex))

        try:  # Try to create new work elements for open requests
            self.logger.info("Adding new elements to open requests")
            extraWork = self.addNewElementsToOpenRequests(queue)
            msg += "Work added: %d, " % extraWork
        except Exception as ex:
            errorMsg = "Generic error while adding work to open requests. Details: %s"
            self.logger.exception(errorMsg, str(ex))

        try:  # Pull in work for new requests
            self.logger.info("Queuing work for new requests")
            work = self.queueNewRequests(queue)
            msg += "New Work: %d" % work
        except Exception as ex:
            errorMsg = "Generic error while queuing work for new requests. Details: %s"
            self.logger.exception(errorMsg, str(ex))

        self.logger.info("Summary of %s: %s", self.__class__.__name__, msg)
        queue.backend.recordTaskActivity('reqmgr_sync', msg)

    def queueNewRequests(self, queue):
        """Get requests from regMgr and queue to workqueue"""
        try:
            workLoads = self.getAvailableRequests()
        except Exception as exc:
            msg = "Error contacting RequestManager. Details: %s" % str(exc)
            self.logger.exception(msg)
            return 0

        work = 0
        for team, reqName, workLoadUrl in workLoads:
            try:
                try:
                    Lexicon.couchurl(workLoadUrl)
                except Exception as ex:  # can throw many errors e.g. AttributeError, AssertionError etc.
                    # check its not a local file
                    if not os.path.exists(workLoadUrl):
                        error = WorkQueueWMSpecError(None, "Workflow url validation error: %s" % str(ex))
                        raise error

                self.logger.info("Processing request %s at %s" % (reqName, workLoadUrl))
                units = queue.queueWork(workLoadUrl, request=reqName, team=team)
                self.logdb.delete(reqName, "error", this_thread=True, agent=False)
            except TERMINAL_EXCEPTIONS as ex:
                # fatal error - report back to ReqMgr
                self.logger.critical('Permanent failure processing request "%s": %s' % (reqName, str(ex)))
                self.logger.info("Marking request %s as failed in ReqMgr" % reqName)
                self.reportRequestStatus(reqName, 'Failed', message=str(ex))
                continue
            except (IOError, socket.error, CouchError, CouchConnectionError) as ex:
                # temporary problem - try again later
                msg = 'Error processing request "%s": will try again later.' % reqName
                msg += '\nError: "%s"' % str(ex)
                self.logger.error(msg)
                self.logdb.post(reqName, msg, 'error')
                continue
            except Exception as ex:
                # Log exception as it isnt a communication problem
                msg = 'Error processing request "%s": will try again later.' % reqName
                msg += '\nSee log for details.\nError: "%s"' % str(ex)
                self.logger.exception('Unknown error processing %s' % reqName)
                self.logdb.post(reqName, msg, 'error')
                continue

            self.logger.info('%s units(s) queued for "%s"' % (units, reqName))
            work += units

        self.logger.info("Total of %s element(s) queued for new requests", work)
        return work

    def cancelWork(self, queue):
        requests = self.reqMgr2.getRequestByStatus(['aborted', 'force-complete'], detail=False)
        count = 0
        for req in requests:
            try:
                queue.cancelWork(req)
                count += 1
            except CouchNotFoundError as exc:
                msg = 'Failed to cancel workflow: {} because elements are no '.format(req)
                msg += 'longer exist in CouchDB. Details: {}'.format(str(exc))
            except Exception as ex:
                msg = 'Error to cancel the request "%s": %s' % (req, str(ex))
                self.logger.exception(msg)
        return count

    def deleteFinishedWork(self, queue, elements):
        """Delete work from queue that is finished in ReqMgr"""
        finished = []
        for element in elements:
            if element.inEndState():
                finished.append(element['RequestName'])
        return queue.deleteWorkflows(*finished)

    def getAvailableRequests(self):
        """
        Get available requests and sort by team and priority
        returns [(team, request_name, request_spec_url)]
        """
        thisStatus = "staged"
        self.logger.info("Contacting ReqMgr for workflows in status: %s", thisStatus)
        tempResults = self.reqMgr2.getRequestByStatus(thisStatus)
        filteredResults = []
        for requests in tempResults:
            for request in viewvalues(requests):
                filteredResults.append(request)
        filteredResults.sort(key=itemgetter('RequestPriority'), reverse=True)
        filteredResults.sort(key=lambda r: r["Team"])

        results = [(x["Team"], x["RequestName"], x["RequestWorkflow"]) for x in filteredResults]

        return results

    def reportRequestStatus(self, request, status, message=None):
        """Change state in RequestManager
           Optionally, take a message to append to the request
        """
        if message:
            logType = "error" if status == "Failed" else "info"
            self.logdb.post(request, str(message), logType)
        reqmgrStatus = self._workQueueToReqMgrStatus(status)

        if reqmgrStatus:  # only send known states
            try:
                self.reqMgr2.updateRequestStatus(request, reqmgrStatus)
            except Exception as ex:
                msg = "%s : fail to update status will try later: %s" % (request, str(ex))
                msg += traceback.format_exc()
                self.logdb.post(request, msg, 'warning')
                raise ex
        return

    def _workQueueToReqMgrStatus(self, status):
        """Map WorkQueue Status to that reported to ReqMgr"""
        statusMapping = {'Acquired': 'acquired',
                         'Running': 'running-open',
                         'Failed': 'failed',
                         'Canceled': 'aborted',
                         'CancelRequested': 'aborted',
                         'Done': 'completed'
                         }
        if status in statusMapping:
            # if wq status passed convert to reqmgr status
            return statusMapping[status]
        elif status in REQUEST_STATE_LIST:
            # if reqmgr status passed return reqmgr status
            return status
        else:
            # unknown status
            return None

    def _reqMgrToWorkQueueStatus(self, status):
        """Map ReqMgr status to that in a WorkQueue element, it is not a 1-1 relation"""
        statusMapping = {'acquired': ['Acquired'],
                         'running': ['Running'],
                         'running-open': ['Running'],
                         'running-closed': ['Running'],
                         'failed': ['Failed'],
                         'aborted': ['Canceled', 'CancelRequested'],
                         'force-complete': ['Canceled', 'CancelRequested'],
                         'completed': ['Done']}
        if status in statusMapping:
            return statusMapping[status]
        else:
            return []

    def reportElement(self, element):
        """Report element to ReqMgr"""
        self.reportRequestStatus(element['RequestName'], element['Status'])

    def addNewElementsToOpenRequests(self, queue):
        """
        Add new elements to open requests according to their
        workqueue_inbox element.
        """
        self.logger.info("Fetching open requests from WorkQueue")

        try:
            workInbox = queue.backend.getInboxElements(OpenForNewData=True, loadSpec=True)
        except Exception as exc:
            self.logger.exception("Error retrieving open inbox elements. Details: %s", str(exc))
            return 0

        self.logger.info("Retrieved %d inbox elements open for new data", len(workInbox))
        work = 0
        for elem in workInbox:
            try:
                units = queue.addWork(elem, rucioObj=self.rucio)
                self.logdb.delete(elem['RequestName'], 'error', True, agent=False)
            except (WorkQueueWMSpecError, WorkQueueNoWorkError) as ex:
                # fatal error - but at least it was split the first time. Log and skip.
                msg = 'Error adding further work to request "%s". ' % elem['RequestName']
                msg += 'Will try again later.\nError: "%s"' % str(ex)
                self.logger.error(msg)
                self.logdb.post(elem['RequestName'], msg, 'error')
            except (IOError, socket.error, CouchError, CouchConnectionError) as ex:
                # temporary problem - try again later
                msg = 'Error processing request "%s": will try again later.' % elem['RequestName']
                msg += '\nError: "%s"' % str(ex)
                self.logger.error(msg)
                self.logdb.post(elem['RequestName'], msg, 'error')
            except Exception as ex:
                # Log exception as it isnt a communication problem
                msg = 'Error processing request "%s". Will try again later. ' % elem['RequestName']
                msg += 'See log for details.\nError: "%s"' % str(ex)
                msg += '\nTraceback: %s' % traceback.format_exc()
                self.logger.exception('Unknown error processing %s' % elem['RequestName'])
                self.logdb.post(elem['RequestName'], msg, 'error')
            else:
                work += units

        self.logger.info("%s element(s) added to open requests" % work)
        return work



#!/usr/bin/env python
from __future__ import division, print_function

import unittest

import logging

from WMCore.ReqMgr.CherryPyThreads.StatusChangeTasks import moveForwardStatus
from WMCore.ReqMgr.DataStructs.RequestError import InvalidStateTransition
from WMCore.ReqMgr.DataStructs.RequestStatus import check_allowed_transition


class MockReqMgr():
    def __init__(self, logger):
        self.logger = logger
        self.currentStatus = None
        self.cacheWflowStatus = {}
        self.logger.info("MockReqMgr: Started local MockReqMgr")

    def getRequestByStatus(self, statusList, detail=True):
        """ Mock the 'getRequestByStatus' method"""
        logging.info("MockReqMgr: getting requests by status: %s, with detail= %s",
                     statusList, detail)
        requestsDict = {"staged": ["wflow1", "wflow2"],
                        "acquired": ["wflow3", "wflow4"],
                        "running-open": ["wflow5", "wflow6"],
                        "running-closed": ["wflow7", "wflow8"],
                        "force-complete": ["wflow9", "wflow10"],
                        "aborted": ["wflow11", "wflow12"]}
        requests = []
        for status in statusList:
            # self.logger.info("MockReqMgr: Setting currentStatus to: %s", status)
            self.currentStatus = status
            if status in requestsDict:
                requests.extend(requestsDict[status])
        return requests

    def updateRequestStatus(self, requestName, statusName):
        """ Mock the 'updateRequestStatus' method"""
        currentStatus = self.cacheWflowStatus.get(requestName, self.currentStatus)
        logging.info("MockReqMgr: updating workflow %s to status: %s",
                     requestName, statusName)
        if check_allowed_transition(currentStatus, statusName):
            self.cacheWflowStatus[requestName] = statusName
            return True
        else:
            raise InvalidStateTransition(requestName, currentStatus, statusName)


class StatusChangeTasksTests(unittest.TestCase):

    def setUp(self):
        """
        Setup the ReqMgr emulator and logger object
        """
        self.logger = logging.getLogger()
        self.reqmgr = MockReqMgr(self.logger)

    def tearDown(self):
        """
        Nothing to tear down
        """
        pass

    def testMoveForwardStatus(self):
        """
        Test the 'moveForwardStatus' function of the CherryPy thread
        An exception should be raised if any invalid status transition is attempted
        """
        # no workflows to perform status transition
        statusFromWQE = dict()
        moveForwardStatus(self.reqmgr, statusFromWQE, self.logger)

        # reqmgr2 and workqueue status in sync, no transition!
        self.reqmgr.cacheWflowStatus.clear()
        statusFromWQE = dict(wflow3='acquired', wflow5='running-open')
        moveForwardStatus(self.reqmgr, statusFromWQE, self.logger)

        # FIXME: an invalid case? I guess workqueue marks it as failed
        # wflow1 transition: staged --> acquired
        # wflow3 transition: acquired --> failed
        self.reqmgr.cacheWflowStatus.clear()
        statusFromWQE = dict(wflow1='failed', wflow3='failed')
        moveForwardStatus(self.reqmgr, statusFromWQE, self.logger)

        # workflow in aborted with completed GQEs, it goes to aborted-completed
        self.reqmgr.cacheWflowStatus.clear()
        statusFromWQE = dict(wflow11='completed', wflow12='completed')
        moveForwardStatus(self.reqmgr, statusFromWQE, self.logger)

        # transition all the way from staged -> acquired -> running-open -> running-closed -> completed
        # and acquired -> running-open -> running-closed
        self.reqmgr.cacheWflowStatus.clear()
        statusFromWQE = dict(wflow1='completed', wflow2='completed',
                             wflow3='running-closed', wflow4='running-closed')
        moveForwardStatus(self.reqmgr, statusFromWQE, self.logger)

        # transition from acquired -> running-open
        self.reqmgr.cacheWflowStatus.clear()
        statusFromWQE = dict(wflow3='running-open', wflow4='running-open')
        moveForwardStatus(self.reqmgr, statusFromWQE, self.logger)


if __name__ == '__main__':
    unittest.main()



from __future__ import (print_function, division)
import unittest

from mock import mock

from WMCore.Storage.Backends.GFAL2Impl import GFAL2Impl


class GFAL2ImplTest(unittest.TestCase):
    def setUp(self):
        self.GFAL2Impl = GFAL2Impl()
        self.removeCommand = self.GFAL2Impl.removeCommand = "removeCommand {}"
        self.copyCommand = self.GFAL2Impl.copyCommand = "copyCommand {checksum} {options} {source} {destination}"

    def testInit(self):
        testGFAL2Impl = GFAL2Impl()
        removeCommand = "env -i X509_USER_PROXY=$X509_USER_PROXY JOBSTARTDIR=$JOBSTARTDIR bash -c " \
                "'. $JOBSTARTDIR/startup_environment.sh; date; gfal-rm -t 600 {}'"
        copyCommand = "env -i X509_USER_PROXY=$X509_USER_PROXY JOBSTARTDIR=$JOBSTARTDIR bash -c '" \
              ". $JOBSTARTDIR/startup_environment.sh; date; gfal-copy -t 2400 -T 2400 -p " \
              "-v --abort-on-failure {checksum} {options} {source} {destination}'"
        self.assertEqual(removeCommand, testGFAL2Impl.removeCommand)
        self.assertEqual(copyCommand, testGFAL2Impl.copyCommand)

    def testCreateSourceName_file(self):
        self.assertEqual("file://name", self.GFAL2Impl.createSourceName("protocol", "file://name"))

    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.os.path')
    def testCreateSourceName_isfile(self, mock_path):
        mock_path.isfile.return_value = True
        mock_path.abspath.return_value = "/some/path"
        self.assertEqual("file:///some/path", self.GFAL2Impl.createSourceName("protocol", "name"))

    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.os.path')
    def testCreateSourceName_startsSlash(self, mock_path):
        mock_path.isfile.return_value = False
        mock_path.abspath.return_value = "/some/path"
        self.assertEqual("file:///some/path", self.GFAL2Impl.createSourceName("protocol", "/name"))

    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.os.path')
    def testCreateSourceName_none(self, mock_path):
        mock_path.isfile.return_value = False
        self.assertEqual("name", self.GFAL2Impl.createSourceName("protocol", "name"))

    def testCreateTargetName_file(self):
        self.assertEqual("file://name", self.GFAL2Impl.createTargetName("protocol", "file://name"))

    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.os.path')
    def testCreateTargetName_isfile(self, mock_path):
        mock_path.isfile.return_value = True
        mock_path.abspath.return_value = "/some/path"
        self.assertEqual("file:///some/path", self.GFAL2Impl.createTargetName("protocol", "name"))

    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.os.path')
    def testCreateTargetName_startsSlash(self, mock_path):
        mock_path.isfile.return_value = False
        mock_path.abspath.return_value = "/some/path"
        self.assertEqual("file:///some/path", self.GFAL2Impl.createTargetName("protocol", "/name"))

    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.os.path')
    def testCreateTargetName_none(self, mock_path):
        mock_path.isfile.return_value = False
        self.assertEqual("name", self.GFAL2Impl.createTargetName("protocol", "name"))

    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.os.path')
    def testCreateRemoveFileCommand_isFile(self, mock_path):
        mock_path.abspath.return_value = "/some/path"
        mock_path.isfile.return_value = True
        self.assertEqual("/bin/rm -f /some/path", self.GFAL2Impl.createRemoveFileCommand("name"))
        mock_path.abspath.assert_called_with("name")

    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.os.path')
    def testCreateRemoveFileCommand_removeCommand(self, mock_path):
        mock_path.isfile.return_value = False
        self.assertEqual("removeCommand file:name", self.GFAL2Impl.createRemoveFileCommand("file:name"))
        mock_path.isfile.assert_called_with("file:name")

    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.GFAL2Impl.createRemoveFileCommand')
    def testCreateStageOutCommand_stageIn(self, mock_createRemoveFileCommand):
        self.GFAL2Impl.stageIn = True
        mock_createRemoveFileCommand.return_value = "targetPFN2"
        result = self.GFAL2Impl.createStageOutCommand("sourcePFN", "targetPFN")
        expectedResult = self.getStageOutCommandResult(
            self.getCopyCommandDict("-K adler32", "", "sourcePFN", "targetPFN"), "targetPFN2")
        mock_createRemoveFileCommand.assert_called_with("targetPFN")
        self.assertEqual(expectedResult, result)

    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.GFAL2Impl.createRemoveFileCommand')
    def testCreateStageOutCommand_options(self, mock_createRemoveFileCommand):
        mock_createRemoveFileCommand.return_value = "targetPFN2"
        result = self.GFAL2Impl.createStageOutCommand("file:sourcePFN", "file:targetPFN", "--nochecksum unknow")
        expectedResult = self.getStageOutCommandResult(
            self.getCopyCommandDict("", "unknow", "file:sourcePFN", "file:targetPFN"), "targetPFN2")
        mock_createRemoveFileCommand.assert_called_with("file:targetPFN")
        self.assertEqual(expectedResult, result)

    def getCopyCommandDict(self, checksum, options, source, destination):
        copyCommandDict = {'checksum': '', 'options': '', 'source': '', 'destination': ''}
        copyCommandDict['checksum'] = checksum
        copyCommandDict['options'] = options
        copyCommandDict['source'] = source
        copyCommandDict['destination'] = destination
        return copyCommandDict

    def getStageOutCommandResult(self, copyCommandDict, createRemoveFileCommandResult):
        result = "#!/bin/bash\n"

        copyCommand = self.copyCommand.format_map(copyCommandDict)
        result += copyCommand

        result += """
            EXIT_STATUS=$?
            echo "gfal-copy exit status: $EXIT_STATUS"
            if [[ $EXIT_STATUS != 0 ]]; then
                echo "ERROR: gfal-copy exited with $EXIT_STATUS"
                echo "Cleaning up failed file:"
                {remove_command}
            fi
            exit $EXIT_STATUS
            """.format(remove_command=createRemoveFileCommandResult)
        
        return result
    
    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.os.path')
    @mock.patch('WMCore.Storage.StageOutImpl.StageOutImpl.executeCommand')
    def testRemoveFile_isFile(self, mock_executeCommand, mock_path):
        mock_executeCommand.return_value = "nu nx"
        mock_path.isfile.return_value = True
        mock_path.abspath.return_value = "/some/file"
        self.GFAL2Impl.removeFile("file")
        mock_path.abspath.assert_called_with("file")
        mock_executeCommand.assert_called_with("/bin/rm -f /some/file")

    @mock.patch('WMCore.Storage.Backends.GFAL2Impl.os.path')
    @mock.patch('WMCore.Storage.StageOutImpl.StageOutImpl.executeCommand')
    def testRemoveFile(self, mock_executeCommand, mock_path):
        mock_path.isfile.return_value = False
        self.GFAL2Impl.removeFile("file://name")
        mock_executeCommand.assert_called_with("removeCommand file://name")



from __future__ import (print_function, division)

import unittest

from mock import mock, call

from WMCore.Storage.Backends.SRMV2Impl import SRMV2Impl
from WMCore.Storage.StageOutError import StageOutError


class SRMV2ImplTest(unittest.TestCase):
    def setUp(self):
        self.SRMV2Impl = SRMV2Impl()

    def testCreateSourceName_startswithSlash(self):
        self.assertEqual("file:////name", self.SRMV2Impl.createSourceName("protocol", "/name"))

    @mock.patch('WMCore.Storage.Backends.SRMV2Impl.os.path')
    def testCreateSourceName_isfile(self, mock_path):
        mock_path.isfile.return_value = True
        mock_path.abspath.return_value = "some/path"
        self.assertEqual("file:///some/path", self.SRMV2Impl.createSourceName("protocol", "name"))

    def testCreateSourceName_simple(self):
        self.assertEqual("name", self.SRMV2Impl.createSourceName("protocol", "name"))

    @mock.patch('WMCore.Storage.Backends.SRMV2Impl.os')
    def testCreateOutputDirectory_stageIn(self, mock_os):
        self.SRMV2Impl.stageIn = True
        mock_os.path.dirname.return_value = "dirName"
        mock_os.path.exists.return_value = True
        self.SRMV2Impl.createOutputDirectory("name")
        mock_os.path.dirname.assert_called_with("name")
        mock_os.path.dirname.exists("name")

        mock_os.path.exists.return_value = False
        self.SRMV2Impl.createOutputDirectory("name")
        mock_os.path.dirname.assert_called_with("name")
        mock_os.path.dirname.exists("name")
        mock_os.makedirs.assert_called_with("dirName")

    @mock.patch('WMCore.Storage.Backends.SRMV2Impl.SRMV2Impl.run')
    def testCreateOutputDirectory_Exception(self, mock_run):
        mock_run.return_value = Exception("I'm in test")
        self.SRMV2Impl.createOutputDirectory("/folder/test/test2/test3/test4/test5")
        calls = [call("srmls -recursion_depth=0 -retry_num=1 /folder/test/test2/test3/test4")]
        mock_run.assert_has_calls(calls)

    @mock.patch('WMCore.Storage.Backends.SRMV2Impl.SRMV2Impl.run')
    def testCreateOutputDirectory_SRM_FAILURE(self, mock_run):
        mock_run.return_value = (0, "SRM_FAILURE test")
        self.SRMV2Impl.createOutputDirectory("folder/test1/test2/test3/test4/test5/test6/test7")
        calls = [call("srmls -recursion_depth=0 -retry_num=1 folder/test1/test2/test3/test4/test5/test6"),
                 call("srmls -recursion_depth=0 -retry_num=1 folder/test1/test2/test3/test4/test5"),
                 call("srmmkdir -retry_num=0 folder/test1/test2/test3/test4/test5/test6")]
        mock_run.assert_has_calls(calls)

    @mock.patch('WMCore.Storage.Backends.SRMV2Impl.SRMV2Impl.run')
    def testCreateOutputDirectory_exitCode(self, mock_run):
        mock_run.return_value = (1, "test")
        self.assertIsNone(self.SRMV2Impl.createOutputDirectory("/folder/test/test2/test3/test4/test5"))
        calls = [call("srmls -recursion_depth=0 -retry_num=1 /folder/test/test2/test3/test4")]
        mock_run.assert_has_calls(calls)

    @mock.patch('WMCore.Storage.Backends.SRMV2Impl.SRMV2Impl.run')
    def testCreateOutputDirectory_exitCode2(self, mock_run):
        mock_run.side_effect = [(0, "SRM_FAILURE test"), (0, "SRM_FAILURE test"), (1, "test")]
        self.assertIsNone(self.SRMV2Impl.createOutputDirectory("folder/test1/test2/test3/test4/test5/test6/test7"))
        calls = [call("srmls -recursion_depth=0 -retry_num=1 folder/test1/test2/test3/test4/test5/test6"),
                 call("srmls -recursion_depth=0 -retry_num=1 folder/test1/test2/test3/test4/test5"),
                 call("srmmkdir -retry_num=0 folder/test1/test2/test3/test4/test5/test6")]
        mock_run.assert_has_calls(calls)

    def testCreateStageOutCommand_stageIn(self):
        self.SRMV2Impl.stageIn = True
        result = self.SRMV2Impl.createStageOutCommand("srm://sourcePFN/test", "file://targetPFN")
        expectedResult = self.geCreateStageOutCommandResults("srm://sourcePFN/test", "file://targetPFN",
                                                             "srm://sourcePFN/test", "targetPFN"
                                                             , "/test", "sourcePFN", "/bin/rm -f targetPFN")
        self.assertEqual(expectedResult, result)

    def testCreateStageOutCommand_options(self):
        result = self.SRMV2Impl.createStageOutCommand("file://sourcePFN/test", "srm://targetPFN/test", options="test")
        expectedResult = self.geCreateStageOutCommandResults("file://sourcePFN/test", "srm://targetPFN/test",
                                                             "srm://targetPFN/test", "sourcePFN/test"
                                                             , "/test", "targetPFN",
                                                             "srmrm -2 -retry_num=0 srm://targetPFN/test",
                                                             options="test")
        self.assertEqual(expectedResult, result)

    def testCreateStageOutCommand_SPF(self):
        result = self.SRMV2Impl.createStageOutCommand("file://sourcePFN/test", "srm://targetPFN/test?SFN=testSPF")
        expectedResult = self.geCreateStageOutCommandResults("file://sourcePFN/test",
                                                             "srm://targetPFN/test?SFN=testSPF",
                                                             "srm://targetPFN/test?SFN=testSPF", "sourcePFN/test"
                                                             , "testSPF", "targetPFN",
                                                             "srmrm -2 -retry_num=0 srm://targetPFN/test?SFN=testSPF")
        self.assertEqual(expectedResult, result)

    @mock.patch('WMCore.Storage.Backends.SRMV2Impl.SRMV2Impl.createRemoveFileCommand')
    def testCreateStageOutCommand_Exception(self, mock_createRemoveFileCommand):
        mock_createRemoveFileCommand.return_value = "test"
        with self.assertRaises(StageOutError):
            self.SRMV2Impl.createStageOutCommand("sourcePFN", "targetPFN")
        mock_createRemoveFileCommand.assert_called_with("targetPFN")

    def geCreateStageOutCommandResults(self, sourcePFN, targetPFN, remotePFN, localPFN, remotePath, remoteHost,
                                       createRemoveFileCommandResult, options=None):
        result = "#!/bin/sh\n"
        result += "REPORT_FILE=`pwd`/srm.report.$$\n"
        result += "srmcp -2 -report=$REPORT_FILE -retry_num=0 -request_lifetime=2400"

        if options != None:
            result += " %s " % options
        result += " %s " % sourcePFN
        result += " %s" % targetPFN
        result += " 2>&1 | tee srm.output.$$ \n"

        result += """
            EXIT_STATUS=`cat $REPORT_FILE | cut -f3 -d" "`
            echo "srmcp exit status: $EXIT_STATUS"
            if [[ "X$EXIT_STATUS" == "X" ]] && [[ `grep -c SRM_INVALID_PATH srm.output.$$` != 0 ]]; then
                echo "ERROR: srmcp failed with SRM_INVALID_PATH"
                exit 1   # dir does not exist
            elif [[ $EXIT_STATUS != 0 ]]; then
               echo "ERROR: srmcp exited with $EXIT_STATUS"
               echo "Cleaning up failed file:"
               %s
               exit $EXIT_STATUS
            fi

            """ % createRemoveFileCommandResult
        result += "FILE_SIZE=`stat -c %s"
        result += " %s `\n" % localPFN
        result += "echo \"Local File Size is: $FILE_SIZE\"\n"

        metadataCheck = \
            """
            SRM_OUTPUT=`srmls -recursion_depth=0 -retry_num=1 %s 2>/dev/null`
            SRM_SIZE=`echo $SRM_OUTPUT | grep '%s' | grep -v '%s' | awk '{print $1;}'`
            echo "SRM Size is $SRM_SIZE"
            if [[ $SRM_SIZE == $FILE_SIZE ]]; then
               exit 0
            else
               echo $SRM_OUTPUT
               echo "ERROR: Size Mismatch between local and SE. Cleaning up failed file..."
               %s
               exit 60311
            fi
            """ % (remotePFN, remotePath, remoteHost, createRemoveFileCommandResult)
        result += metadataCheck

        return result

    @mock.patch('WMCore.Storage.StageOutImpl.StageOutImpl.executeCommand')
    def testRemoveFile(self, mock_executeCommand):
        self.SRMV2Impl.removeFile("file")
        mock_executeCommand.assert_called_with("srmrm -2 -retry_num=0 file")



from builtins import object
from future.utils import viewitems, viewvalues

import os
import cherrypy
import logging
from functools import wraps

from WMCore.REST.Main import RESTMain
from WMCore.REST.Test import fake_authz_headers, fake_authz_key_file
from WMCore.Services.Requests import JSONRequests

class RESTMainTestServer(object):

    def __init__(self, cfg, statedir, testName):
        self.server = RESTMain(cfg, statedir)
        self.testName = testName
        self.config = cfg
        self.port = cfg.main.port
        self.host = '127.0.0.1'
        self.serverUrl = "http://%s:%s/%s/" % (self.host, self.port, cfg.main.application)

        ## test authentication using fake filepermission
        self.test_authz_key = fake_authz_key_file(False)
        self.config.main.tools.cms_auth.key_file = self.test_authz_key.name
        #self.header = fake_authz_headers(test_authz_key.data, roles = {"Admin": {'group': ['ReqMgr']}})

        self.jsonSender = JSONRequests(self.serverUrl)

    def getLastTest(self):
        global lastTest
        return lastTest

    def setLastTest(self):
        global lastTest
        lastTest = self.testName


    def start(self, blocking = True):
        self.server.validate_config()
        self.server.setup_server()
        self.server.install_application()
        cherrypy.config.update({'server.socket_port': self.port})
        cherrypy.config.update({'server.socket_host': self.host})
        cherrypy.config.update({'request.show_tracebacks': True})
        cherrypy.config.update({'environment': 'test_suite'})
        cherrypy.config.update({'server.thread_pool': 10})
        cherrypy.config.update({'server.accepted_queue_size': -1})
        cherrypy.config.update({'server.accepted_queue_timeout': 10})
        for app in viewvalues(cherrypy.tree.apps):
            if '/' in app.config:
                app.config["/"]["request.show_tracebacks"] = True

        cherrypy.server.httpserver = None
        cherrypy.engine.start()
        if blocking:
            cherrypy.engine.block()


    def stop(self):
        """
        Stop the server
        """
        cherrypy.engine.exit()
        cherrypy.engine.stop()

        # Ensure the next server that's started gets fresh objects
        for name, server in viewitems(getattr(cherrypy, 'servers', {})):
            server.unsubscribe()
            del cherrypy.servers[name]

        self.test_authz_key.close()
        os.remove(self.test_authz_key.name)



"""
File       : MSPileupTasks_t.py
Author     : Valentin Kuznetsov <vkuznet AT gmail dot com>
Description: Unit tests for MicorService/MSPileup/MSPileupReport.py module
"""

# system modules
import unittest

# WMCore modules
from WMCore.MicroService.MSPileup.DataStructs.MSPileupReport import MSPileupReport


class MSPileupReportTest(unittest.TestCase):
    """Unit test for MSPileupTasks module"""

    def testMSPileupReport(self):
        """
        Test MSPileup report functionality
        """
        mgr = MSPileupReport(autoCleanup=True)
        entry = 'test'
        task = 'tast'
        uuid = '123'
        mgr.addEntry(task, uuid, entry)

        # check documents functionality
        docs = mgr.getDocuments()
        self.assertEqual(len(docs), 1)
        self.assertEqual(docs[0]['entry'], entry)
        self.assertEqual(docs[0]['task'], task)
        self.assertEqual(docs[0]['uuid'], uuid)

        # check dictionary funtionality
        rdict = mgr.getReportByUuid()
        self.assertEqual(uuid in rdict, True)
        self.assertEqual(len(rdict[uuid]), 1)
        self.assertEqual(entry in rdict[uuid][0], True)
        self.assertEqual(uuid in rdict[uuid][0], True)
        self.assertEqual(task in rdict[uuid][0], True)

        # test purge functionality
        mgr = MSPileupReport(autoExpire=-1, autoCleanup=True)
        mgr.addEntry(task, uuid, entry)
        docs = mgr.getDocuments()
        self.assertEqual(len(docs), 0)


if __name__ == '__main__':
    unittest.main()



# pylint: disable=E0239
# E0239: inherit-non-class
"""
File       : Data.py
Author     : Valentin Kuznetsov <vkuznet AT gmail dot com>
Description: This module consists of all REST APIs required by MicroService

Client may invoke GET/POST calls to MicroService, e.g.
# return status of the MicroService
curl http://localhost:8822/microservice/data/status
# ping MicroService, i.e. return its default message
curl http://localhost:8822/microservice/data
# post data to MicroService
curl -X POST -H "Content-type: application/json" -d '{"request":{"spec":"spec"}}' http://localhost:8822/microservice/data

# MSPileup get APIs
# spec.json will contains spec query to delete MSPileup document, e.g.
# {"query": {"pileupName": "bla-bla-bla"}}
curl -X POST -H "Content-type: application/json" -d@./spec.json http://lhost:port/ms-pileup/data

or we will use dedicated end-point, e.g. data
curl -X GET http://lhost:port/ms-pileup/data/pileup?pileupName=bla
curl -X GET http://lhost:port/ms-pileup/data/pileup?campaign=campaign

# MSPileup create APIs
# doc.json contains new MSPileup document
curl -X POST -H "Content-type: application/json" -d@./doc.json http://lhost:port/ms-pileup/data

# MSPileup update API
curl -X PUT -H "Content-type: application/json" -d '{"pileupName":"bla"}' http://localhost:8822/microservice/data

# MSPileup delete API
curl -X DELETE -H "Content-type: application/json" -d '{"pileupName":"bla"}' http://localhost:8822/microservice/data
"""
# futures
from __future__ import print_function, division

# system modules
import json
import traceback
import importlib
from future.utils import with_metaclass
# from types import GeneratorType

# 3d party modules
import cherrypy

# WMCore modules
import WMCore
from Utils.Patterns import Singleton
from WMCore.REST.Server import RESTEntity, restcall
from WMCore.REST.Tools import tools
# from WMCore.REST.Validation import validate_rx, validate_str
from WMCore.REST.Format import JSONFormat, PrettyJSONFormat


def results(res):
    "Return results in a list format suitable by REST server"
    if not isinstance(res, list):
        return [res]
    return res


def mspileupError(doc):
    """
    Check MSPileup record for error
    :return: None or raise cherrypy.HTTPError
    """
    if 'error' in doc:
        msg = doc['message']
        code = doc['code']
        msg = f'MSPileupError: {msg}, code: {code}'
        raise cherrypy.HTTPError(400, msg) from None


class Data(with_metaclass(Singleton, RESTEntity, object)):
    """
    This class is responsbiel for both the REST interface
    and the application/service itself, thus make it a
    Singleton to guarantee that only one instance will be
    executing
    """
    def __init__(self, app, api, config, mount):
        RESTEntity.__init__(self, app, api, config, mount)
        self.config = config
        arr = config.manager.split('.')
        self.mount = mount
        try:
            cname = arr[-1]
            module = importlib.import_module('.'.join(arr[:-1]))
            self.mgr = getattr(module, cname)(config)
        except ImportError:
            print("ERROR initializing MicroService REST module.")
            traceback.print_exc()

    def validate(self, apiobj, method, api, param, safe):
        """
        Validate request input data.
        Has to be implemented, otherwise the service fails to start.
        If it's not implemented correctly (e.g. just pass), the arguments
        are not passed in the method at all.

        """
        if method == 'GET':
            for prop in list(param.kwargs):
                safe.kwargs[prop] = param.kwargs.pop(prop)
            safe.kwargs['API'] = api
            if param.args:
                return False
        elif method == 'POST':
            if not param.args or not param.kwargs:
                return False
        return True

    @restcall(formats=[('text/plain', PrettyJSONFormat()), ('application/json', JSONFormat())])
    @tools.expires(secs=-1)
    def get(self, **kwds):
        """
        Implement GET request with given uid or set of parameters
        """
        res = {'wmcore_version': WMCore.__version__,
               'microservice_version': WMCore.__version__,  # FIXME: extract it from another place
               'microservice': self.mgr.__class__.__name__}

        api = kwds.get('API')

        if api == "status":
            detail = True if kwds.pop("detail", True) in (True, "true", "True", "TRUE") else False
            res.update(self.mgr.status(detail, **kwds))
            yield res
        elif api == "info":
            res.update(self.mgr.info(kwds.pop("request", None), **kwds))
            yield res
        elif 'ms-pileup' in self.mount:
            # this section of the GET API is only relevant for MSPileup service
            # the self.mount point should match service configuration, e.g. in MSPileup we define
            # mount point as /ms-pileup/data, e.g. http://.../ms-pileup/data
            try:
                for doc in self.mgr.get(**kwds):
                    mspileupError(doc)
                    yield doc
            except cherrypy.HTTPError:
                raise
            except:
                msg = 'Unable to GET request, error=%s' % traceback.format_exc()
                raise cherrypy.HTTPError(status=500, message=msg) from None

    @restcall(formats=[('application/json', JSONFormat())])
    @tools.expires(secs=-1)
    def post(self):
        """
        Implement POST request API.
        The input HTTP request should be in the following form
        {"request":some_data} for posting the data into the service.

        NOTE: the usage of this method requires further thought
        """
        try:
            data = json.load(cherrypy.request.body)
            if 'ms-pileup' in self.mount:
                # this section of POST API is only relevant for MSPileup service
                for doc in self.mgr.post(data):
                    mspileupError(doc)
                    yield doc
            elif 'ms-transferor' in self.mount:
                for doc in self.mgr.post(data):
                    yield doc
            else:
                msg = f"End point {self.mount} does not support POST request JSON payload"
                raise cherrypy.HTTPError(status=400, message=msg) from None
        except cherrypy.HTTPError:
            raise
        except:
            msg = 'Unable to POST request, error=%s' % traceback.format_exc()
            raise cherrypy.HTTPError(status=500, message=msg) from None

    @restcall(formats=[('application/json', JSONFormat())])
    @tools.expires(secs=-1)
    def put(self):
        """
        Implement PUT HTTP method of MicroService API.
        """
        try:
            if 'ms-pileup' in self.mount:
                # this section of PUT API is only relevant for MSPileup service
                data = json.load(cherrypy.request.body)
                for doc in self.mgr.update(data):
                    mspileupError(doc)
                    yield doc
        except cherrypy.HTTPError:
            raise
        except:
            msg = 'Unable to PUT request, error=%s' % traceback.format_exc()
            raise cherrypy.HTTPError(status=500, message=msg) from None

    @restcall(formats=[('application/json', JSONFormat())])
    @tools.expires(secs=-1)
    def delete(self):
        """
        Implement DELETE HTTP method of MicroService API.
        """
        if 'ms-pileup' not in self.mount:
            return # not implemented for other MS services

        # DELETE API is only relevant for MSPileup service
        spec = {}
        try:
            spec = json.load(cherrypy.request.body)
        except:
            msg = 'Unable to DELETE request, error=%s' % traceback.format_exc()
            raise cherrypy.HTTPError(status=400, message=msg) from None
        # so far allow only pileupName in input spec
        if 'pileupName' not in spec:
            msg = "wrong spec, please provide pileupName"
            raise cherrypy.HTTPError(status=400, message=msg) from None
        try:
            if 'pileupName' in spec and len(spec.keys()) == 1:
                for doc in self.mgr.delete(spec):
                    mspileupError(doc)
                    yield doc
        except cherrypy.HTTPError:
            raise
        except:
            msg = 'Unable to DELETE request, error=%s' % traceback.format_exc()
            raise cherrypy.HTTPError(status=500, message=msg) from None



#!/usr/bin/env python

import os
import json
import shutil
import logging
import tarfile
import tempfile
import unittest

from WMComponent.WorkflowUpdater.WorkflowUpdaterPoller import \
    blockLocations, checkChanges, updateBlockInfo, writePileupJson, \
    findJsonSandboxFiles, extractPileupconf, WorkflowUpdaterPoller
from WMCore.MicroService.Tools.Common import getMSLogger
from WMCore.WMBase import getTestBase
from WMCore.Configuration import Configuration
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase
from WMQuality.TestInit import TestInit


class WorkflowUpdaterPollerTest(EmulatedUnitTestCase):
    """
    Test case for the WorkflowUpdaterPoller
    """

    def setUp(self):
        """
        Setup the database and logging connection. Try to create all of the
        WMBS tables.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()

        self.logger = getMSLogger(False)
        super(WorkflowUpdaterPollerTest, self).setUp()

        # setup WorkflowUpdater
        config = Configuration()
        config.component_('WorkflowUpdater')
        self.dbsUrl = 'https://cmsweb-testbed.cern.ch/dbs/int/global/DBSReader'
        config.WorkflowUpdater.dbsUrl = self.dbsUrl
        config.WorkflowUpdater.rucioAccount = 'wma_test'
        config.WorkflowUpdater.rucioUrl = 'http://cms-rucio-int.cern.ch'
        config.WorkflowUpdater.rucioAuthUrl = 'https://cms-rucio-auth-int.cern.ch'
        config.WorkflowUpdater.rucioCustomScope = 'group.wmcore'
        config.WorkflowUpdater.msPileupUrl = 'https://cmsweb-testbed.cern.ch/mspileup'
        self.obj = WorkflowUpdaterPoller(config)
        self.obj.logger = self.logger

        # set data objects we will use through the test
        testData = os.path.join(getTestBase(), "WMComponent_t/WorkflowUpdater_t")
        sandbox1 = 'SC_MultiPU_Agent227_Val_240110_215719_7133-Sandbox.tar.bz2'
        self.sandbox1 = os.path.join(testData, sandbox1)
        self.jdoc = {"mc": {"block1": {"FileList": [], "NumberOfEvents": 1, "PhEDExNodeNames": ['rse1']},
                            "block2": {"FileList": [], "NumberOfEvents": 1, "PhEDExNodeNames": ['rse2']},
                            "block3": {"FileList": [], "NumberOfEvents": 1, "PhEDExNodeNames": ['rse3']}}}

    def testFindJsonSandboxFiles(self):
        """
        Test findJsonSandboxFiles function
        """
        files = findJsonSandboxFiles(self.sandbox1)
        self.assertEqual(len(files), 2)
        self.logger.info(files)
        expect = ['WMSandbox/GenSimFull/cmsRun1/pileupconf.json', 'WMSandbox/GenSimFull/cmsRun2/pileupconf.json']
        self.assertEqual(files, expect)

    def testExtractPileupconf(self):
        """
        Test extractPileupconf function
        """
        self.logger.info(self.sandbox1)
        files = findJsonSandboxFiles(self.sandbox1)
        for fname in files:
            jdoc = extractPileupconf(self.sandbox1, fname)
            keys = list(jdoc.keys())
            self.assertEqual(keys, ['mc'])
            bdict = blockLocations(jdoc)
            blocks = bdict.keys()
            self.logger.info("%s found %d block names", fname, len(blocks))
            if fname == 'WMSandbox/GenSimFull/cmsRun1/pileupconf.json':
                self.assertEqual(len(blocks), 1)
            elif fname == 'WMSandbox/GenSimFull/cmsRun2/pileupconf.json':
                self.assertEqual(len(blocks), 463)

    def testBlockLocations(self):
        """
        Test blockLocations function
        """
        bdict = blockLocations(dict(self.jdoc))
        self.assertEqual(list(bdict.keys()), ["block1", "block2", "block3"])
        self.assertEqual(bdict['block1'], ['rse1'])
        self.assertEqual(bdict['block2'], ['rse2'])
        self.assertEqual(bdict['block3'], ['rse3'])

    def testUpdataBlockInfo(self):
        """
        Test updateBlockInfo function
        """
        jdoc = dict(self.jdoc)
        rses = ['rse1', 'rse2', 'rse3']
        binfo = {'PhEDExNodeNames': rses, 'files': [], 'events': 0}
        msPUBlockLoc = {'block1': binfo, 'block2': binfo}
        doc = updateBlockInfo(jdoc, msPUBlockLoc, self.dbsUrl, self.logger)
        for rec in doc.values():
            # for block1 and block2 we expect new list of rses from blockInfo
            # block3 should be discarded by now
            self.assertEqual(len(rec), 2)
            self.assertEqual(rec['block1']['PhEDExNodeNames'], binfo)
            self.assertEqual(rec['block2']['PhEDExNodeNames'], binfo)

    def testCheckChanges(self):
        """
        Test checkChanges function
        """
        jdict = dict(self.jdoc)['mc']

        # test first use-case, i.e. block in json conf has different rses from MSPileup
        msPUBlockLoc = blockLocations(dict(self.jdoc))
        # update our dict that it will have different rses
        msPUBlockLoc['block1'] = []
        status = checkChanges(jdict, msPUBlockLoc)
        self.assertEqual(status, True)

        # test second use-case, i.e. we have extra blocks in MSPileup than present in json conf
        msPUBlockLoc = blockLocations(dict(self.jdoc))
        msPUBlockLoc['block-extra'] = []
        status = checkChanges(jdict, msPUBlockLoc)
        self.assertEqual(status, True)

    def testWritePileupJson(self):
        """
        Test writePileupJson function
        """
        testDoc = {"test": True, "int": 1, "list": [1, 2, 3]}
        jdict = {}
        tmpFile = '/tmp/WMComponent-Sandbox.tar.bz2'

        # write new content to tmpFile
        files = findJsonSandboxFiles(self.sandbox1)
        for fname in files:
            jdoc = extractPileupconf(self.sandbox1, fname)
            # we skip updating part and write our test doc
            jdict[fname] = testDoc
        self.logger.info("Write %s to %s", jdict, tmpFile)
        writePileupJson(self.sandbox1, jdict, self.logger, tmpFile)

        # now we extract from tmpFile our pileupconf.json files and make comparison of their content
        for fname in files:
            jdoc = extractPileupconf(tmpFile, fname)
            self.logger.info("Extracted %s from %s (%s)", jdoc, tmpFile, fname)
            self.assertEqual(jdoc, testDoc)

        # remove tmpFile
        os.remove(tmpFile)

    def testAdjustJSONSpec(self):
        """
        Test adjustJSONSpec function
        """
        self.obj.logger.setLevel(logging.DEBUG)

        # extract workflow name from our test tar.bz2
        fname = os.path.basename(self.sandbox1)
        wflowName = fname.replace(".tar.bz2", "").replace("-Sandbox", "")

        # copy our test tar.bz2 file to test area
        dstDir = os.path.join('/tmp', wflowName)
        self.logger.debug("### create %s", dstDir)
        if not os.path.exists(dstDir):
            os.makedirs(dstDir)
        tarFileName = os.path.join(dstDir, fname)
        self.logger.debug("### copy %s to %s", self.sandbox1, dstDir)
        shutil.copyfile(self.sandbox1, tarFileName)

        # prepare our pileup information using information from self.sandbox1
        # actual wflowSpec example
        # /storage/local/data1/cmsdataops/srv/wmagent/v2.3.0/install/wmagentpy3/WorkQueueManager/cache/wangz_task_BPH-RunIISummer20UL18GEN-00263__v1_T_240106_151823_5508/WMSandbox/WMWorkload.pkl
        # we fake it as following since we keep our stuff in /tmp area
        wflowSpec = "/tmp/SC_MultiPU_Agent227_Val_240110_215719_7133/WMSandbox/WMWorkload.pkl"

        pileup1 = "/RelValMinBias_14TeV/CMSSW_11_2_0_pre8-112X_mcRun3_2024_realistic_v10_forTrk-v1/GEN-SIM"
        pileup1Blocks = ["/RelValMinBias_14TeV/CMSSW_11_2_0_pre8-112X_mcRun3_2024_realistic_v10_forTrk-v1/GEN-SIM#b9304f4c-5efd-49e5-bb85-c1f15eb4a1ad"]
        pileup2 = "/Neutrino_E-10_gun/RunIISummer20ULPrePremix-UL16_106X_mcRun2_asymptotic_v13-v1/PREMIX"
        pileup2Blocks = ["/Neutrino_E-10_gun/RunIISummer20ULPrePremix-UL16_106X_mcRun2_asymptotic_v13-v1/PREMIX#e1c457b0-b1f5-4951-9f7b-64cd81276697",
                         "/Neutrino_E-10_gun/RunIISummer20ULPrePremix-UL16_106X_mcRun2_asymptotic_v13-v1/PREMIX#a401e768-1a28-49e8-b478-6ed7ad29b5bb"]
        pileups = {pileup1: pileup1Blocks, pileup2: pileup2Blocks}

        # untar our sandbox file and adjust pileup blocks
        with tarfile.open(tarFileName, 'r:bz2') as tar:
            tar.extractall(path=dstDir)
            self.logger.debug("### files in %s: %s", dstDir, os.listdir(dstDir))

            # here we process our sandbox content with new pileup info
            for pileup, blocks in pileups.items():
                # our new pileup workflow list
                puWflows = [{'name': wflowName, 'spec': wflowSpec, 'pileup': pileup}]

                # let's fake MSPlieup list with fake rses and blocks
                msPileupList = [{'pileupName': pileup, 'rses': ["rse1"], "blocks": blocks}]
                self.logger.debug("### unit test adjustJSONSpec puWflows=%s msPileupList=%s", puWflows, msPileupList)

                # perform adjustment of JSON files in sandbox tarball
                self.obj.adjustJSONSpec(puWflows, msPileupList)

        # at this point we should updated one of our configuration files and it should have empty content
        # here we process our sandbox content with new pileup info
        with tarfile.open(tarFileName, 'r:bz2') as tar:
            with tempfile.TemporaryDirectory() as tmpDir:
                tar.extractall(path=tmpDir)

                # test 1: in first pileup we should see rses change
                confName = os.path.join(tmpDir, 'WMSandbox/GenSimFull/cmsRun1/pileupconf.json')
                blocks = pileups[pileup1]
                # please note: the confName json files has two blocks
                # since we specified only one block in blocks list (see above)
                # we should only get one block in new content
                with open(confName, 'r', encoding='utf-8') as istream:
                    content = json.load(istream)
                    # get number of blocks in new content
                    jdict = content['mc']
                    self.logger.debug("### test confName   %s", confName)
                    self.logger.debug("### orig jsonBlocks %s", blocks)
                    self.logger.debug("### new msPUBlocks  %s", list(jdict.keys()))
                    self.assertEqual(len(jdict.keys()), 1)
                    self.assertEqual(jdict[blocks[0]]["PhEDExNodeNames"], ["rse1"])

                confName = os.path.join(tmpDir, 'WMSandbox/GenSimFull/cmsRun2/pileupconf.json')
                blocks = pileups[pileup2]
                # please note: the confName json files has two blocks
                # since we specified only one block in blocks list (see above)
                # we should only get one block in new content
                with open(confName, 'r', encoding='utf-8') as istream:
                    content = json.load(istream)
                    # get number of blocks in new content
                    jdict = content['mc']
                    self.logger.debug("### test confName   %s", confName)
                    self.logger.debug("### orig jsonBlocks %s", blocks)
                    self.logger.debug("### new msPUBlocks  %s", list(jdict.keys()))
                    # since we only requested on block in blocks list it should be discarded in new content
                    self.assertEqual(len(jdict.keys()), 2)
                    self.assertEqual(jdict[blocks[0]]["PhEDExNodeNames"], ["rse1"])
                    self.assertEqual(jdict[blocks[1]]["PhEDExNodeNames"], ["rse1"])

        # reset logger back to original
        self.obj.logger.setLevel(logging.WARNING)
        self.logger.setLevel(logging.WARNING)

        # clean-up destination area
        shutil.rmtree(dstDir)


if __name__ == "__main__":
    unittest.main()



"""
Module defines REST API methods and their handles.
Implementation of handles is in corresponding modules, not here.

"""
from __future__ import (division, print_function)
import cherrypy

from WMCore.Configuration import Configuration
from WMCore.REST.Server import RESTApi
from WMCore.REST.Format import JSONFormat

from WMCore.WMStats.Service.MetaDataInfo import ServerInfo
from WMCore.WMStats.Service.ActiveRequestJobInfo import ActiveRequestJobInfo
from WMCore.WMStats.Service.RequestInfo import JobDetailInfo

class T0RestApiHub(RESTApi):
    """
    Server object for REST data access API.

    """
    def __init__(self, app, config, mount):
        """
        :arg app: reference to application object; passed to all entities.
        :arg config: reference to configuration; passed to all entities.
        :arg str mount: API URL mount point; passed to all entities."""

        RESTApi.__init__(self, app, config, mount)

        cherrypy.log("T0WMStats entire configuration:\n%s" % Configuration.getInstance())
        cherrypy.log("T0WMStats REST hub configuration subset:\n%s" % config)
        # only allows json format for return value
        self.formats =  [('application/json', JSONFormat())]
        self._add({"info": ServerInfo(app, self, config, mount),
                   "requestcache": ActiveRequestJobInfo(app, self, config, mount),
                   "jobdetail": JobDetailInfo(app, self, config, mount, t0flag=True),
                  })



from __future__ import print_function
import logging
from future import standard_library
standard_library.install_aliases()
import urllib.request

import unittest
from nose.plugins.attrib import attr

import os
from functools import reduce
from hashlib import md5

from WMCore.WMSpec.Steps.Executors.DQMUpload import DQMUpload

class StepPatch():
    def __init__(self, upload=None):
        self.upload = upload

class UploadPatch():
    def __init__(self):
        self.proxy = False

class DQMUpload_t(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    @attr("integration")
    def test_upload(self):
        """
        test_upload

        This unittest has been used to validate the modernization of DQMUpload.
        If everything is ok, this unittest is expected to pass.

        However, this unittest sends a 144MB root file to the DQM. We convened 
        that it is a bit too much for multiple daily checks, so we mark this
        as "integration". See discussion at:
        https://github.com/dmwm/WMCore/pull/9934#issuecomment-898855725
        """
        filename = "DQM_V0001_R000000001__RelValH125GGgluonfusion_13__CMSSW_8_1_0-RecoFullPU_2017PU_TaskChain_PUMCRecyc_HG1705_Validation_TEST_Alan_v67-v11__DQMIO.root"
        filepath = os.path.join("/tmp", filename)
        if not os.path.isfile(filepath):
            urllib.request.urlretrieve(
                "http://amaltaro.web.cern.ch/amaltaro/forAlan/%s" % filename, 
                filepath)

        dqm = DQMUpload()
        upload_patch = UploadPatch()
        step_patch = StepPatch(upload_patch)
        dqm.step = step_patch

        args = {}
        # Preparing a checksum
        blockSize = 0x10000
        def upd(m, data):
            m.update(data)
            return m
        with open(filepath, 'rb') as fd:
            contents = iter(lambda: fd.read(blockSize), b'')
            m = reduce(upd, contents, md5())
        args['checksum'] = 'md5:%s' % m.hexdigest()
        args['size'] = os.path.getsize(filepath)
        headers, data = dqm.upload(
            "https://cmsweb-testbed.cern.ch/dqm/dev/", 
            args,
            filepath)

        logging.debug("headers: %s %s", type(headers), headers)
        logging.debug("data: %s %s", type(data), data)

        # Dqm-Status-Code == 100 is considered good
        # Dqm-Status-Code == 300 was related to " File name does not match the expected convention"
        self.assertEqual(headers.get("Dqm-Status-Code", None), '100')

    @attr("integration")
    def test_httppost(self):
        """
        test_httppost

        This unittest has been used to validate the modernization of DQMUpload.
        If everything is ok, this unittest is expected to pass.
        """
        filename = "DQM_V0001_R000000001__RelValH125GGgluonfusion_13__CMSSW_8_1_0-RecoFullPU_2017PU_TaskChain_PUMCRecyc_HG1705_Validation_TEST_Alan_v67-v11__DQMIO.root"
        filepath = os.path.join("/tmp", filename)
        if not os.path.isfile(filepath):
            urllib.request.urlretrieve(
                "http://amaltaro.web.cern.ch/amaltaro/forAlan/%s" % filename, 
                filepath)

        dqm = DQMUpload()
        upload_patch = UploadPatch()
        step_patch = StepPatch(upload_patch)
        dqm.step = step_patch
        dqm.step.upload.URL = "https://cmsweb-testbed.cern.ch/dqm/dev/"
        dqm.httpPost(filepath)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_SetSite_

Oracle implementation of JobGroup.SetSite
"""

__all__ = []



from WMCore.WMBS.MySQL.JobGroup.SetSite import SetSite as MySQLSetSite

class SetSite(MySQLSetSite):
    """
    Right now, same as MySQL

    """



'''
Created on Dec 16, 2016

'''
from __future__ import (division, print_function)

import unittest

from WMCore.Services.TagCollector.TagCollector import TagCollector


class TagCollectorTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        """
        # using the default production server
        self.tagCollecor = TagCollector()
        return

    def testTagCollecorMethods(self):
        """
        _testTagCollecorMethods_
        """
        releases = self.tagCollecor.releases()
        architectures = self.tagCollecor.architectures()
        realsese_by_arch = self.tagCollecor.releases_by_architecture()
        microarch_by_release = self.tagCollecor.defaultMicroArchVersionNumberByRelease()
        microarch_testCMSSW_15 = self.tagCollecor.getGreaterMicroarchVersionNumber("CMSSW_15_0_0_pre3")
        microarch_testCMSSW_12_15 = self.tagCollecor.getGreaterMicroarchVersionNumber("CMSSW_12_4_0_pre2,CMSSW_15_0_0_pre3")
        microarch_testCMSSW_7_12 = self.tagCollecor.getGreaterMicroarchVersionNumber("CMSSW_7_1_10_patch2,CMSSW_12_4_0_pre2", rel_microarchs=microarch_by_release)
        self.assertIn('CMSSW_7_1_10_patch2', releases)
        self.assertIn('slc6_amd64_gcc493', architectures)
        self.assertIn('slc7_amd64_gcc620', architectures)
        self.assertEqual(len(architectures), len(realsese_by_arch))
        self.assertEqual(sorted(self.tagCollecor.releases('slc6_amd64_gcc493')),
                         sorted(realsese_by_arch.get('slc6_amd64_gcc493')))
        self.assertEqual(0, microarch_by_release['CMSSW_7_1_10_patch2'])
        self.assertEqual(3, microarch_by_release['CMSSW_15_0_0_pre3'])
        self.assertEqual(3, microarch_testCMSSW_12_15)
        self.assertEqual(3, microarch_testCMSSW_15)
        self.assertEqual(0, microarch_testCMSSW_7_12)

        return


if __name__ == '__main__':
    unittest.main()



"""
Unit tests for MSPileupMonitoring.py module

Author: Valentin Kuznetsov <vkuznet [AT] gmail [DOT] com>
"""

# system modules
import unittest

# WMCore modules
from WMCore.MicroService.MSPileup.MSPileupMonitoring import flatDocuments, flatKey


class MSPileupMonitoringTest(unittest.TestCase):
    "Unit test for MSPileupMonitoring module"

    def setUp(self):
        rses = ['rse1', 'rse2']
        campaigns = ['c1', 'c2']
        ruleIds = ['1', '2']
        self.doc = {
            'pileupName': '/klsjdfklsd/klsjdflksdj/PREMIX',
            'pileupType': 'classic',
            'expectedRSEs': rses,
            'currentRSEs': rses,
            'fullReplicas': 1,
            'campaigns': campaigns,
            'containerFraction': 1.0,
            'replicationGrouping': "ALL",
            'active': True,
            'pileupSize': 0,
            'ruleIds': ruleIds}

    def testFlatKey(self):
        "test flatKey functions"
        doc = dict(self.doc)
        docs = list(flatKey(doc, 'campaigns'))
        self.assertEqual(len(docs), 2)
        key = 'campaigns'
        nkey = key[:-1]  # new single key, e.g. campaigns -> campaign
        self.assertEqual(key in docs[0], False)
        self.assertEqual(docs[0][nkey], self.doc[key][0])

    def testFlatDocuments(self):
        "test flatDocuments function"
        doc = dict(self.doc)
        docs = list(flatDocuments(doc))
        self.assertEqual(len(docs), 8)
        listKeys = ['campaigns', 'currentRSEs', 'expectedRSEs']
        for doc in docs:
            for key in listKeys:
                vals = self.doc[key]  # original key vaules
                nkey = key[:-1]  # key without s, e.g. campaigns -> campaign
                self.assertEqual(key in doc, False)  # original key should be gone
                val = doc[nkey]  # new value for single key
                self.assertEqual(val in vals, True)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_DestroyAll_

"""

from WMCore.Database.DBFormatter import DBFormatter

class New(DBFormatter):
    tableSql = "SELECT table_name FROM user_tables"
    seqSql = "SELECT sequence_name FROM user_sequences"
    trgSql = "SELECT trigger_name FROM user_triggers"

    def execute(self, conn = None, transaction = False):
        tbl = self.dbi.processData(self.tableSql, conn = conn, transaction = transaction)
        seq = self.dbi.processData(self.seqSql, conn = conn, transaction = transaction)
        trg = self.dbi.processData(self.trgSql, conn = conn, transaction = transaction)

        for table in self.formatDict(tbl):
            self.dbi.processData("DROP TABLE %s CASCASE CONSTRAINTS" % table["table_name"],
                                 conn = conn, transaction = transaction)

        for sequence in self.formatDict(seq):
            self.dbi.processData("DROP SEQUENCE %s" % sequence["sequence_name"],
                                 conn = conn, transaction = transaction)

        for trigger in self.formatDict(trg):
            self.dbi.processData("DROP TTRIGGER %s" % trigger["trigger_name"],
                                 conn = conn, transaction = transaction)



#!/usr/bin/env python
"""
_DeleteFiles_

Builder implementation for DeleteFiles

"""
from __future__ import print_function
from WMCore.WMSpec.Steps.Builder import Builder
from WMCore.WMSpec.ConfigSectionTree import nodeName





class DeleteFiles(Builder):
    """
    _DeleteFiles_

    Build a working area for a DeleteFiles step

    """

    def build(self, step, workingDir, **args):
        """
        _build_

        implement build interface for DeleteFiles Step

        """
        stepName = nodeName(step)
        stepWorkingArea = "%s/%s" % (workingDir, stepName)
        self.installWorkingArea(step, stepWorkingArea)
        print("Builders.DeleteFiles.build called on %s" % stepName)



#!/usr/bin/env python
"""
_JobPackage_

Unittests for JobPackage persistency mechanism
"""

from builtins import range
import os
import unittest

from WMQuality.TestInit import TestInit

from WMCore.DataStructs.JobPackage import JobPackage
from WMCore.DataStructs.Job import Job

class JobPackageTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Create a temporary file to presist the JobPackage to.
        """
        self.testInit = TestInit(__file__)
        self.persistFile = os.path.join(self.testInit.generateWorkDir(),
                                        "JobPackage.pkl")
        return

    def tearDown(self):
        pass

    def testAddingJobs(self):
        """
        _testAddingJobs_

        Verify that adding jobs to the package works as expected.
        """
        package = JobPackage()

        for i in range(100):
            newJob = Job("Job%s" % i)
            newJob["id"] = i
            package[i] = newJob

        # There is an extra key for the directory the package is stored in.
        assert len(package) == 101, \
               "Error: Wrong number of jobs in package."

        for i in range(100):
            job = package[i]
            assert job["id"] == i, \
                   "Error: Jobs has wrong ID."
            assert job["name"] == "Job%d" % i, \
                   "Error: Job has wrong name."

        return

    def testPersist(self):
        """
        _testPersist_

        Verify that we're able to save and load the job package.
        """
        package = JobPackage()

        for i in range(100):
            newJob = Job("Job%s" % i)
            newJob["id"] = i
            package[i] = newJob

        package.save(self.persistFile)

        assert os.path.exists(self.persistFile), \
               "Error: Package file was never created."

        newPackage = JobPackage()
        newPackage.load(self.persistFile)

        # There is an extra key for the directory the package is stored in.
        assert len(newPackage) == 101, \
               "Error: Wrong number of jobs in package."

        for i in range(100):
            job = newPackage[i]
            assert job["id"] == i, \
                   "Error: Jobs has wrong ID."
            assert job["name"] == "Job%d" % i, \
                   "Error: Job has wrong name."

        return

    def testBaggage(self):
        """
        _testBaggage_

        Verify that job baggage is persisted with the package.
        """
        package = JobPackage()

        for i in range(100):
            newJob = Job("Job%s" % i)
            newJob["id"] = i
            baggage = newJob.getBaggage()
            setattr(baggage, "thisJob", newJob["name"])
            setattr(baggage, "seed1", 11111111)
            setattr(baggage, "seed2", 22222222)
            setattr(baggage, "seed3", 33333333)
            setattr(baggage, "seed4", 44444444)
            setattr(baggage, "seed5", 55555555)
            package[i] = newJob

        package.save(self.persistFile)

        assert os.path.exists(self.persistFile), \
               "Error: Package file was never created."

        newPackage = JobPackage()
        newPackage.load(self.persistFile)

        # There is an extra key for the directory the package is stored in.
        assert len(newPackage) == 101, \
               "Error: Wrong number of jobs in package."

        for i in range(100):
            job = newPackage[i]
            assert job["id"] == i, \
                   "Error: Jobs has wrong ID."
            assert job["name"] == "Job%d" % i, \
                   "Error: Job has wrong name."
            jobBaggage = job.getBaggage()

            assert jobBaggage.thisJob == "Job%d" % i, \
                   "Error: Job baggage has wrong name."
            assert jobBaggage.seed1 == 11111111, \
                   "Error: Job baggee has wrong value for seed1."
            assert jobBaggage.seed2 == 22222222, \
                   "Error: Job baggee has wrong value for seed2."
            assert jobBaggage.seed3 == 33333333, \
                   "Error: Job baggee has wrong value for seed3."
            assert jobBaggage.seed4 == 44444444, \
                   "Error: Job baggee has wrong value for seed4."
            assert jobBaggage.seed5 == 55555555, \
                   "Error: Job baggee has wrong value for seed5."


        return

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
TaskArchiver test

Tests both the archiving of tasks and the creation of the
workloadSummary
"""
from __future__ import division
from builtins import range

import json
import logging
import os.path
import re
import threading
import time
import unittest

from WMCore_t.WMSpec_t.TestSpec import createTestWorkload
from nose.plugins.attrib import attr

import WMCore.WMBase
from WMComponent.TaskArchiver.CleanCouchPoller import CleanCouchPoller
from WMComponent.TaskArchiver.TaskArchiverPoller import TaskArchiverPoller
from WMCore.DAOFactory import DAOFactory
from WMCore.DataStructs.Run import Run
from WMCore.Database.CMSCouch import CouchServer
from WMCore.FwkJobReport.Report import Report
from WMCore.JobStateMachine.ChangeState import ChangeState
from WMCore.Lexicon import sanitizeURL
from WMCore.Services.RequestDB.RequestDBWriter import RequestDBWriter
from WMCore.Services.UUIDLib import makeUUID
from WMCore.WMBS.File import File
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Job import Job
from WMCore.WMBS.JobGroup import JobGroup
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMCore.WMBase import getTestBase
from WMCore.WMSpec.Makers.TaskMaker import TaskMaker
from WMQuality.Emulators.WMSpecGenerator.ReqMgrDocGenerator import generate_reqmgr_schema
from WMQuality.TestInitCouchApp import TestInitCouchApp as TestInit
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase


class TaskArchiverTest(EmulatedUnitTestCase):
    """
    TestCase for TestTaskArchiver module
    """

    _setup_done = False
    _teardown = False
    _maxMessage = 10
    OWNERDN = os.environ['OWNERDN'] if 'OWNERDN' in os.environ else "Generic/OWNERDN"

    def setUp(self):
        """
        setup for test.
        """
        super(TaskArchiverTest, self).setUp()
        myThread = threading.currentThread()

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection(destroyAllDatabase=True)
        self.testInit.setSchema(customModules=["WMCore.WMBS", "WMComponent.DBS3Buffer"],
                                useDefault=False)
        self.databaseName = "taskarchiver_t_0"
        self.testInit.setupCouch("%s/workloadsummary" % self.databaseName, "WorkloadSummary")
        self.testInit.setupCouch("%s/jobs" % self.databaseName, "JobDump")
        self.testInit.setupCouch("%s/fwjrs" % self.databaseName, "FWJRDump")
        self.testInit.setupCouch("wmagent_summary_t", "WMStats")
        self.testInit.setupCouch("wmagent_summary_central_t", "WMStats")
        self.testInit.setupCouch("stat_summary_t", "SummaryStats")
        reqmgrdb = "reqmgrdb_t"
        self.testInit.setupCouch(reqmgrdb, "ReqMgr")

        reqDBURL = "%s/%s" % (self.testInit.couchUrl, reqmgrdb)
        self.requestWriter = RequestDBWriter(reqDBURL)
        self.requestWriter.defaultStale = {}

        self.daofactory = DAOFactory(package="WMCore.WMBS",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)

        self.dbsDaoFactory = DAOFactory(package="WMComponent.DBS3Buffer",
                                        logger=myThread.logger,
                                        dbinterface=myThread.dbi)

        self.getJobs = self.daofactory(classname="Jobs.GetAllJobs")
        self.inject = self.daofactory(classname="Workflow.MarkInjectedWorkflows")

        self.testDir = self.testInit.generateWorkDir()
        os.makedirs(os.path.join(self.testDir, 'specDir'))

        self.nJobs = 10
        self.campaignName = 'aCampaign'

        return

    def tearDown(self):
        """
        Database deletion
        """

        self.testInit.clearDatabase(modules=["WMCore.WMBS"])
        self.testInit.delWorkDir()
        self.testInit.tearDownCouch()
        return

    def getConfig(self):
        """
        _createConfig_

        General config file
        """
        config = self.testInit.getConfiguration()
        # self.testInit.generateWorkDir(config)

        config.section_("General")
        config.General.workDir = "."
        config.General.ReqMgr2ServiceURL = "https://cmsweb-dev.cern.ch/reqmgr2"

        config.section_("JobStateMachine")
        config.JobStateMachine.couchurl = os.getenv("COUCHURL", "cmssrv52.fnal.gov:5984")
        config.JobStateMachine.couchDBName = self.databaseName
        config.JobStateMachine.jobSummaryDBName = 'wmagent_summary_t'
        config.JobStateMachine.summaryStatsDBName = 'stat_summary_t'

        config.component_("JobCreator")
        config.JobCreator.jobCacheDir = os.path.join(self.testDir, 'testDir')

        config.component_("TaskArchiver")
        config.TaskArchiver.componentDir = self.testDir
        config.TaskArchiver.WorkQueueParams = {'CacheDir': config.JobCreator.jobCacheDir}
        config.TaskArchiver.pollInterval = 60
        config.TaskArchiver.logLevel = 'INFO'
        config.TaskArchiver.timeOut = 0
        config.TaskArchiver.histogramKeys = ['AvgEventTime', 'writeTotalMB', 'jobTime']
        config.TaskArchiver.histogramBins = 5
        config.TaskArchiver.histogramLimit = 5
        config.TaskArchiver.perfPrimaryDatasets = ['SingleMu', 'MuHad', 'MinimumBias']
        config.TaskArchiver.perfDashBoardMinLumi = 50
        config.TaskArchiver.perfDashBoardMaxLumi = 9000
        config.TaskArchiver.dqmUrl = 'https://cmsweb.cern.ch/dqm/dev/'
        config.TaskArchiver.dashBoardUrl = 'http://dashboard43.cern.ch/dashboard/request.py/putluminositydata'
        config.TaskArchiver.workloadSummaryCouchDBName = "%s/workloadsummary" % self.databaseName
        config.TaskArchiver.localWMStatsURL = "%s/%s" % (config.JobStateMachine.couchurl,
                                                         config.JobStateMachine.jobSummaryDBName)
        config.TaskArchiver.workloadSummaryCouchURL = config.JobStateMachine.couchurl
        config.TaskArchiver.requireCouch = True

        config.component_("AnalyticsDataCollector")
        config.AnalyticsDataCollector.centralRequestDBURL = '%s/reqmgrdb_t' % config.JobStateMachine.couchurl
        config.AnalyticsDataCollector.RequestCouchApp = "ReqMgr"

        config.section_("ACDC")
        config.ACDC.couchurl = config.JobStateMachine.couchurl
        config.ACDC.database = config.JobStateMachine.couchDBName

        # Make the jobCacheDir
        os.mkdir(config.JobCreator.jobCacheDir)

        # addition for Alerts messaging framework, work (alerts) and control
        # channel addresses to which the component will be sending alerts
        # these are destination addresses where AlertProcessor:Receiver listens
        config.section_("Alert")
        config.Alert.address = "tcp://127.0.0.1:5557"
        config.Alert.controlAddr = "tcp://127.0.0.1:5559"

        config.section_("Agent")
        config.Agent.serverDN = "/we/bypass/myproxy/logon"

        return config

    def createWorkload(self, workloadName):
        """
        Creates a test workload for us to run on, hold the basic necessities.
        """

        workload = createTestWorkload(workloadName)

        taskMaker = TaskMaker(workload, os.path.join(self.testDir, 'workloadTest'))
        taskMaker.skipSubscription = True
        taskMaker.processWorkload()

        workload.setCampaign(self.campaignName)

        workload.save(workloadName)

        return workload

    def createTestJobGroup(self, config, name="TestWorkthrough",
                           filesetName="TestFileset",
                           specLocation="spec.xml", error=False,
                           task="/TestWorkload/ReReco",
                           jobType="Processing"):
        """
        Creates a group of several jobs

        """

        testWorkflow = Workflow(spec=specLocation, owner=self.OWNERDN,
                                name=name, task=task, owner_vogroup="", owner_vorole="")
        testWorkflow.create()
        self.inject.execute(names=[name], injected=True)

        testWMBSFileset = Fileset(name=filesetName)
        testWMBSFileset.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10)
        testFileA.addRun(Run(10, *[12312]))
        testFileA.setLocation('malpaquet')

        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=10)
        testFileB.addRun(Run(10, *[12314]))
        testFileB.setLocation('malpaquet')

        testFileA.create()
        testFileB.create()

        testWMBSFileset.addFile(testFileA)
        testWMBSFileset.addFile(testFileB)
        testWMBSFileset.commit()
        testWMBSFileset.markOpen(0)

        outputWMBSFileset = Fileset(name='%sOutput' % filesetName)
        outputWMBSFileset.create()
        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=10)
        testFileC.addRun(Run(10, *[12312]))
        testFileC.setLocation('malpaquet')
        testFileC.create()
        outputWMBSFileset.addFile(testFileC)
        outputWMBSFileset.commit()
        outputWMBSFileset.markOpen(0)

        testWorkflow.addOutput('output', outputWMBSFileset)

        testSubscription = Subscription(fileset=testWMBSFileset,
                                        workflow=testWorkflow,
                                        type=jobType)
        testSubscription.create()

        testJobGroup = JobGroup(subscription=testSubscription)
        testJobGroup.create()

        for i in range(0, self.nJobs):
            testJob = Job(name=makeUUID())
            testJob.addFile(testFileA)
            testJob.addFile(testFileB)
            testJob['retry_count'] = 1
            testJob['retry_max'] = 10
            testJob['mask'].addRunAndLumis(run=10, lumis=[12312, 12313])
            testJobGroup.add(testJob)

        testJobGroup.commit()

        changer = ChangeState(config)

        report1 = Report()
        report2 = Report()
        if error:
            path1 = os.path.join(WMCore.WMBase.getTestBase(),
                                 "WMComponent_t/JobAccountant_t/fwjrs", "badBackfillJobReport.pkl")
            path2 = os.path.join(WMCore.WMBase.getTestBase(),
                                 'WMComponent_t/TaskArchiver_t/fwjrs',
                                 'logCollectReport2.pkl')
        else:
            path1 = os.path.join(WMCore.WMBase.getTestBase(),
                                 'WMComponent_t/TaskArchiver_t/fwjrs',
                                 'mergeReport1.pkl')
            path2 = os.path.join(WMCore.WMBase.getTestBase(),
                                 'WMComponent_t/TaskArchiver_t/fwjrs',
                                 'logCollectReport2.pkl')
        report1.load(filename=path1)
        report2.load(filename=path2)

        changer.propagate(testJobGroup.jobs, 'created', 'new')
        changer.propagate(testJobGroup.jobs, 'executing', 'created')
        changer.propagate(testJobGroup.jobs, 'complete', 'executing')
        for i in range(self.nJobs):
            if i < self.nJobs // 2:
                testJobGroup.jobs[i]['fwjr'] = report1
            else:
                testJobGroup.jobs[i]['fwjr'] = report2
        changer.propagate(testJobGroup.jobs, 'jobfailed', 'complete')
        changer.propagate(testJobGroup.jobs, 'jobcooloff', 'jobfailed')
        changer.propagate(testJobGroup.jobs, 'created', 'jobcooloff')
        changer.propagate(testJobGroup.jobs, 'executing', 'created')
        changer.propagate(testJobGroup.jobs, 'complete', 'executing')
        changer.propagate(testJobGroup.jobs, 'jobfailed', 'complete')
        changer.propagate(testJobGroup.jobs, 'retrydone', 'jobfailed')
        changer.propagate(testJobGroup.jobs, 'exhausted', 'retrydone')
        changer.propagate(testJobGroup.jobs, 'cleanout', 'exhausted')

        testSubscription.completeFiles([testFileA, testFileB])

        return testJobGroup

    def createGiantJobSet(self, name, config, nSubs=10, nJobs=10,
                          nFiles=1, spec="spec.xml"):
        """
        Creates a massive set of jobs

        """

        jobList = []

        for i in range(0, nSubs):
            # Make a bunch of subscriptions
            localName = '%s-%i' % (name, i)
            testWorkflow = Workflow(spec=spec, owner=self.OWNERDN,
                                    name=localName, task="Test", owner_vogroup="", owner_vorole="")
            testWorkflow.create()

            testWMBSFileset = Fileset(name=localName)
            testWMBSFileset.create()

            testSubscription = Subscription(fileset=testWMBSFileset,
                                            workflow=testWorkflow)
            testSubscription.create()

            testJobGroup = JobGroup(subscription=testSubscription)
            testJobGroup.create()

            filesToComplete = []

            for j in range(0, nJobs):
                # Create jobs for each subscription
                testFileA = File(lfn="%s-%i-lfnA" % (localName, j), size=1024, events=10)
                testFileA.addRun(Run(10, *[11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
                                           21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
                                           31, 32, 33, 34, 35, 36, 37, 38, 39, 40]))
                testFileA.setLocation('malpaquet')
                testFileA.create()

                testWMBSFileset.addFile(testFileA)
                testWMBSFileset.commit()

                filesToComplete.append(testFileA)

                testJob = Job(name='%s-%i' % (localName, j))
                testJob.addFile(testFileA)
                testJob['retry_count'] = 1
                testJob['retry_max'] = 10
                testJobGroup.add(testJob)
                jobList.append(testJob)

                for k in range(0, nFiles):
                    # Create output files
                    testFile = File(lfn="%s-%i-output" % (localName, k), size=1024, events=10)
                    testFile.addRun(Run(10, *[12312]))
                    testFile.setLocation('malpaquet')
                    testFile.create()

                    testJobGroup.output.addFile(testFile)

                testJobGroup.output.commit()

            testJobGroup.commit()

            changer = ChangeState(config)

            changer.propagate(testJobGroup.jobs, 'created', 'new')
            changer.propagate(testJobGroup.jobs, 'executing', 'created')
            changer.propagate(testJobGroup.jobs, 'complete', 'executing')
            changer.propagate(testJobGroup.jobs, 'success', 'complete')
            changer.propagate(testJobGroup.jobs, 'cleanout', 'success')

            testWMBSFileset.markOpen(0)

            testSubscription.completeFiles(filesToComplete)

        return jobList

    def getPerformanceFromDQM(self, dqmUrl, dataset, run):
        # Make function to fetch this from DQM. Returning Null or False if it fails
        getUrl = "%sjsonfairy/archive/%s%s/DQM/TimerService/event_byluminosity" % (dqmUrl, run, dataset)
        # Assert if the URL is assembled as expected
        if run == 207214:
            self.assertEqual(
                'https://cmsweb.cern.ch/dqm/dev/jsonfairy/archive/207214/MinimumBias/Commissioning10-v4/DQM/DQM/TimerService/event_byluminosity',
                getUrl)
        # let's suppose it works..
        testResponseFile = open(os.path.join(getTestBase(),
                                             'WMComponent_t/TaskArchiver_t/DQMGUIResponse.json'), 'r')
        response = testResponseFile.read()
        testResponseFile.close()
        responseJSON = json.loads(response)
        return responseJSON

    def filterInterestingPerfPoints(self, responseJSON, minLumi, maxLumi):
        worthPoints = {}
        points = responseJSON["hist"]["bins"]["content"]
        for i in range(responseJSON["hist"]["xaxis"]["first"]["id"], responseJSON["hist"]["xaxis"]["last"]["id"]):
            # is the point worth it? if yes add to interesting points dictionary.
            # 1 - non 0
            # 2 - between minimum and maximum expected luminosity
            # FIXME : 3 - population in dashboard for the bin interval < 100
            # Those should come from the config :
            if points[i] == 0:
                continue
            binSize = responseJSON["hist"]["xaxis"]["last"]["value"] // responseJSON["hist"]["xaxis"]["last"]["id"]
            # Fetching the important values
            instLuminosity = i * binSize
            timePerEvent = points[i]

            if instLuminosity > minLumi and instLuminosity < maxLumi:
                worthPoints[instLuminosity] = timePerEvent
        return worthPoints

    def publishPerformanceDashBoard(self, dashBoardUrl, PD, release, worthPoints):
        dashboardPayload = []
        for instLuminosity in worthPoints:
            timePerEvent = int(worthPoints[instLuminosity])
            dashboardPayload.append({"primaryDataset": PD,
                                     "release": release,
                                     "integratedLuminosity": instLuminosity,
                                     "timePerEvent": timePerEvent})

        data = "{\"data\":%s}" % str(dashboardPayload).replace("\'", "\"")

        # let's suppose it works..
        testDashBoardPayloadFile = open(os.path.join(getTestBase(),
                                                     'WMComponent_t/TaskArchiver_t/DashBoardPayload.json'), 'r')
        testDashBoardPayload = testDashBoardPayloadFile.read()
        testDashBoardPayloadFile.close()

        self.assertEqual(data, testDashBoardPayload)

        return True

    def populateWorkflowWithCompleteStatus(self, name="TestWorkload"):
        schema = generate_reqmgr_schema(1)
        schema[0]["RequestName"] = name

        self.requestWriter.insertGenericRequest(schema[0])
        result = self.requestWriter.updateRequestStatus(name, "completed")
        return result

    def testA_BasicFunctionTest(self):
        """
        _BasicFunctionTest_

        Tests the components, by seeing if they can process a simple set of closeouts
        """

        myThread = threading.currentThread()

        config = self.getConfig()
        workloadPath = os.path.join(self.testDir, 'specDir', 'spec.pkl')
        workload = self.createWorkload(workloadName=workloadPath)
        testJobGroup = self.createTestJobGroup(config=config,
                                               name=workload.name(),
                                               specLocation=workloadPath,
                                               error=False)

        # Create second workload
        testJobGroup2 = self.createTestJobGroup(config=config,
                                                name=workload.name(),
                                                filesetName="TestFileset_2",
                                                specLocation=workloadPath,
                                                task="/TestWorkload/ReReco/LogCollect",
                                                jobType="LogCollect")

        cachePath = os.path.join(config.JobCreator.jobCacheDir,
                                 "TestWorkload", "ReReco")
        os.makedirs(cachePath)
        self.assertTrue(os.path.exists(cachePath))

        cachePath2 = os.path.join(config.JobCreator.jobCacheDir,
                                  "TestWorkload", "LogCollect")
        os.makedirs(cachePath2)
        self.assertTrue(os.path.exists(cachePath2))

        result = myThread.dbi.processData("SELECT * FROM wmbs_subscription")[0].fetchall()
        self.assertEqual(len(result), 2)

        workflowName = "TestWorkload"
        dbname = config.TaskArchiver.workloadSummaryCouchDBName
        couchdb = CouchServer(config.JobStateMachine.couchurl)
        workdatabase = couchdb.connectDatabase(dbname)
        jobdb = couchdb.connectDatabase("%s/jobs" % self.databaseName)
        fwjrdb = couchdb.connectDatabase("%s/fwjrs" % self.databaseName)
        jobs = jobdb.loadView("JobDump", "jobsByWorkflowName",
                              options={"startkey": [workflowName],
                                       "endkey": [workflowName, {}]})['rows']
        fwjrdb.loadView("FWJRDump", "fwjrsByWorkflowName", options={"startkey": [workflowName],
                                                                    "endkey": [workflowName, {}]})['rows']

        self.assertEqual(len(jobs), 2 * self.nJobs)

        from WMCore.WMBS.CreateWMBSBase import CreateWMBSBase
        create = CreateWMBSBase()
        tables = []
        for x in create.requiredTables:
            tables.append(x[2:])

        self.populateWorkflowWithCompleteStatus()
        testTaskArchiver = TaskArchiverPoller(config=config)
        testTaskArchiver.algorithm()

        cleanCouch = CleanCouchPoller(config=config)
        cleanCouch.setup()
        cleanCouch.algorithm()

        result = myThread.dbi.processData("SELECT * FROM wmbs_job")[0].fetchall()
        self.assertEqual(len(result), 0)
        result = myThread.dbi.processData("SELECT * FROM wmbs_subscription")[0].fetchall()
        self.assertEqual(len(result), 0)
        result = myThread.dbi.processData("SELECT * FROM wmbs_jobgroup")[0].fetchall()
        self.assertEqual(len(result), 0)
        result = myThread.dbi.processData("SELECT * FROM wmbs_fileset")[0].fetchall()
        self.assertEqual(len(result), 0)
        result = myThread.dbi.processData("SELECT * FROM wmbs_file_details")[0].fetchall()
        self.assertEqual(len(result), 0)

        # Make sure we deleted the directory
        self.assertFalse(os.path.exists(cachePath))
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'workloadTest/TestWorkload')))

        testWMBSFileset = Fileset(id=1)
        self.assertEqual(testWMBSFileset.exists(), False)

        workloadSummary = workdatabase.document(id="TestWorkload")
        # Check ACDC
        self.assertEqual(workloadSummary['ACDCServer'], sanitizeURL(config.ACDC.couchurl)['url'])

        # Check the output
        self.assertEqual(list(workloadSummary['output']), ['/Electron/MorePenguins-v0/RECO'])
        self.assertEqual(sorted(workloadSummary['output']['/Electron/MorePenguins-v0/RECO']['tasks']),
                         ['/TestWorkload/ReReco', '/TestWorkload/ReReco/LogCollect'])
        # Check performance
        # Check histograms
        self.assertAlmostEqual(
            workloadSummary['performance']['/TestWorkload/ReReco']['cmsRun1']['AvgEventTime']['histogram'][0][
                'average'],
            0.89405199999999996, places=2)
        self.assertEqual(
            workloadSummary['performance']['/TestWorkload/ReReco']['cmsRun1']['AvgEventTime']['histogram'][0][
                'nEvents'],
            10)

        # Check standard performance
        self.assertAlmostEqual(
            workloadSummary['performance']['/TestWorkload/ReReco']['cmsRun1']['TotalJobCPU']['average'],
            17.786300000000001,
            places=2)
        self.assertAlmostEqual(
            workloadSummary['performance']['/TestWorkload/ReReco']['cmsRun1']['TotalJobCPU']['stdDev'], 0.0,
            places=2)

        # Check worstOffenders
        self.assertEqual(
            workloadSummary['performance']['/TestWorkload/ReReco']['cmsRun1']['AvgEventTime']['worstOffenders'],
            [{'logCollect': None, 'log': None, 'value': '0.894052', 'jobID': 1},
             {'logCollect': None, 'log': None, 'value': '0.894052', 'jobID': 1},
             {'logCollect': None, 'log': None, 'value': '0.894052', 'jobID': 2}])

        # Check retryData
        self.assertEqual(workloadSummary['retryData']['/TestWorkload/ReReco'], {'1': 10})
        logCollectPFN = 'srm://srm-cms.cern.ch:8443/srm/managerv2?SFN=/castor/cern.ch/cms/store/logs/prod/2012/11/WMAgent/Run206446-MinimumBias-Run2012D-v1-Tier1PromptReco-4af7e658-23a4-11e2-96c7-842b2b4671d8/Run206446-MinimumBias-Run2012D-v1-Tier1PromptReco-4af7e658-23a4-11e2-96c7-842b2b4671d8-AlcaSkimLogCollect-1-logs.tar'
        self.assertEqual(workloadSummary['logArchives'],
                         {'/TestWorkload/ReReco/LogCollect': [logCollectPFN for _ in range(10)]})

        # LogCollect task is made out of identical FWJRs
        # assert that it is identical
        for x in workloadSummary['performance']['/TestWorkload/ReReco/LogCollect']['cmsRun1']:
            if x in config.TaskArchiver.histogramKeys:
                continue
            for y in ['average', 'stdDev']:
                self.assertAlmostEqual(
                    workloadSummary['performance']['/TestWorkload/ReReco/LogCollect']['cmsRun1'][x][y],
                    workloadSummary['performance']['/TestWorkload/ReReco']['cmsRun1'][x][y],
                    places=2)

        return

    def testB_testErrors(self):
        """
        _testErrors_

        Test with a failed FWJR
        """

        config = self.getConfig()
        workloadPath = os.path.join(self.testDir, 'specDir', 'spec.pkl')
        workload = self.createWorkload(workloadName=workloadPath)
        testJobGroup = self.createTestJobGroup(config=config,
                                               name=workload.name(),
                                               specLocation=workloadPath,
                                               error=True)
        # Create second workload
        testJobGroup2 = self.createTestJobGroup(config=config,
                                                name=workload.name(),
                                                filesetName="TestFileset_2",
                                                specLocation=workloadPath,
                                                task="/TestWorkload/ReReco/LogCollect",
                                                jobType="LogCollect")

        cachePath = os.path.join(config.JobCreator.jobCacheDir,
                                 "TestWorkload", "ReReco")
        os.makedirs(cachePath)
        self.assertTrue(os.path.exists(cachePath))

        couchdb = CouchServer(config.JobStateMachine.couchurl)
        jobdb = couchdb.connectDatabase("%s/jobs" % self.databaseName)
        fwjrdb = couchdb.connectDatabase("%s/fwjrs" % self.databaseName)
        jobdb.loadView("JobDump", "jobsByWorkflowName",
                       options={"startkey": [workload.name()],
                                "endkey": [workload.name(), {}]})['rows']
        fwjrdb.loadView("FWJRDump", "fwjrsByWorkflowName",
                        options={"startkey": [workload.name()],
                                 "endkey": [workload.name(), {}]})['rows']

        self.populateWorkflowWithCompleteStatus()
        testTaskArchiver = TaskArchiverPoller(config=config)
        testTaskArchiver.algorithm()

        cleanCouch = CleanCouchPoller(config=config)
        cleanCouch.setup()
        cleanCouch.algorithm()

        dbname = getattr(config.JobStateMachine, "couchDBName")
        workdatabase = couchdb.connectDatabase("%s/workloadsummary" % dbname)

        workloadSummary = workdatabase.document(id=workload.name())

        self.assertEqual(workloadSummary['errors']['/TestWorkload/ReReco']['failureTime'], 500)
        self.assertTrue('99999' in workloadSummary['errors']['/TestWorkload/ReReco']['cmsRun1'])

        failedRunInfo = workloadSummary['errors']['/TestWorkload/ReReco']['cmsRun1']['99999']['runs']
        self.assertEqual(failedRunInfo, {'10': [[12312, 12312]]},
                         "Wrong lumi information in the summary for failed jobs")

        # Check the failures by site histograms
        self.assertEqual(
            workloadSummary['histograms']['workflowLevel']['failuresBySite']['data']['T1_IT_CNAF']['Failed Jobs'], 10)
        self.assertEqual(
            workloadSummary['histograms']['stepLevel']['/TestWorkload/ReReco']['cmsRun1']['errorsBySite']['data'][
                'T1_IT_CNAF']['99999'], 10)
        self.assertEqual(
            workloadSummary['histograms']['stepLevel']['/TestWorkload/ReReco']['cmsRun1']['errorsBySite']['data'][
                'T1_IT_CNAF']['8020'], 10)
        self.assertEqual(workloadSummary['histograms']['workflowLevel']['failuresBySite']['average']['Failed Jobs'], 10)
        self.assertEqual(
            workloadSummary['histograms']['stepLevel']['/TestWorkload/ReReco']['cmsRun1']['errorsBySite']['average'][
                '99999'], 10)
        self.assertEqual(
            workloadSummary['histograms']['stepLevel']['/TestWorkload/ReReco']['cmsRun1']['errorsBySite']['average'][
                '8020'], 10)
        self.assertEqual(workloadSummary['histograms']['workflowLevel']['failuresBySite']['stdDev']['Failed Jobs'], 0)
        self.assertEqual(
            workloadSummary['histograms']['stepLevel']['/TestWorkload/ReReco']['cmsRun1']['errorsBySite']['stdDev'][
                '99999'], 0)
        self.assertEqual(
            workloadSummary['histograms']['stepLevel']['/TestWorkload/ReReco']['cmsRun1']['errorsBySite']['stdDev'][
                '8020'], 0)
        return

    @attr("integration")
    def testC_Profile(self):
        """
        _Profile_

        DON'T RUN THIS!
        """
        import cProfile
        import pstats

        name = makeUUID()

        config = self.getConfig()

        jobList = self.createGiantJobSet(name=name, config=config,
                                         nSubs=10, nJobs=1000, nFiles=10)

        cleanCouch = CleanCouchPoller(config=config)
        cleanCouch.setup()

        cProfile.runctx("cleanCouch.algorithm()", globals(), locals(), filename="testStats.stat")

        p = pstats.Stats('testStats.stat')
        p.sort_stats('cumulative')
        p.print_stats()
        return

    @attr("integration")
    def testD_Timing(self):
        """
        _Timing_

        This is to see how fast things go.
        """
        myThread = threading.currentThread()

        name = makeUUID()

        config = self.getConfig()
        jobList = self.createGiantJobSet(name=name, config=config, nSubs=10,
                                         nJobs=1000, nFiles=10)

        testTaskArchiver = TaskArchiverPoller(config=config)

        startTime = time.time()
        testTaskArchiver.algorithm()
        stopTime = time.time()

        result = myThread.dbi.processData("SELECT * FROM wmbs_job")[0].fetchall()
        self.assertEqual(len(result), 0)
        result = myThread.dbi.processData("SELECT * FROM wmbs_subscription")[0].fetchall()
        self.assertEqual(len(result), 0)
        result = myThread.dbi.processData("SELECT * FROM wmbs_jobgroup")[0].fetchall()
        self.assertEqual(len(result), 0)
        result = myThread.dbi.processData("SELECT * FROM wmbs_file_details")[0].fetchall()
        self.assertEqual(len(result), 0)
        testWMBSFileset = Fileset(id=1)
        self.assertEqual(testWMBSFileset.exists(), False)

        logging.info("TaskArchiver took %f seconds", (stopTime - startTime))

    def testDQMRecoPerformanceToDashBoard(self):

        myThread = threading.currentThread()

        listRunsWorkflow = self.dbsDaoFactory(classname="ListRunsWorkflow")

        # Didn't like to have done that, but the test doesn't provide all info I need in the system, so faking it:
        myThread.dbi.processData("""insert into dbsbuffer_workflow(id, name) values (1, 'TestWorkload')""",
                                 transaction=False)
        myThread.dbi.processData(
            """insert into dbsbuffer_file (id, lfn, dataset_algo, workflow) values (1, '/store/t/e/s/t.test', 1, 1)""",
            transaction=False)
        myThread.dbi.processData(
            """insert into dbsbuffer_file (id, lfn, dataset_algo, workflow) values (2, '/store/t/e/s/t.test2', 1, 1)""",
            transaction=False)
        myThread.dbi.processData(
            """insert into dbsbuffer_file_runlumi_map (run, lumi, filename) values (207214, 100, 1)""",
            transaction=False)
        myThread.dbi.processData(
            """insert into dbsbuffer_file_runlumi_map (run, lumi, filename) values (207215, 200, 2)""",
            transaction=False)

        config = self.getConfig()

        dqmUrl = getattr(config.TaskArchiver, "dqmUrl")
        perfDashBoardMinLumi = getattr(config.TaskArchiver, "perfDashBoardMinLumi")
        perfDashBoardMaxLumi = getattr(config.TaskArchiver, "perfDashBoardMaxLumi")
        dashBoardUrl = getattr(config.TaskArchiver, "dashBoardUrl")

        workloadPath = os.path.join(self.testDir, 'specDir', 'spec.pkl')
        workload = self.createWorkload(workloadName=workloadPath)
        testJobGroup = self.createTestJobGroup(config=config,
                                               name=workload.name(),
                                               specLocation=workloadPath,
                                               error=True)
        testJobGroup2 = self.createTestJobGroup(config=config,
                                                name=workload.name(),
                                                filesetName="TestFileset_2",
                                                specLocation=workloadPath,
                                                task="/TestWorkload/ReReco/LogCollect",
                                                jobType="LogCollect")

        # Adding request type as ReReco, real ReqMgr requests have it
        workload.data.request.section_("schema")
        workload.data.request.schema.RequestType = "ReReco"
        workload.data.request.schema.CMSSWVersion = 'test_compops_CMSSW_5_3_6_patch1'
        workload.getTask('ReReco').addInputDataset(name='/a/b/c', primary='a', processed='b', tier='c')

        interestingPDs = getattr(config.TaskArchiver, "perfPrimaryDatasets")
        interestingDatasets = []
        # Are the datasets from this request interesting? Do they have DQM output? One might ask afterwards if they have harvest
        for dataset in workload.listOutputDatasets():
            (nothing, PD, procDataSet, dataTier) = dataset.split('/')
            if PD in interestingPDs and dataTier == "DQM":
                interestingDatasets.append(dataset)
        # We should have found 1 interesting dataset
        self.assertAlmostEqual(len(interestingDatasets), 1)
        if len(interestingDatasets) == 0:
            return
        # Request will be only interesting for performance if it's a ReReco or PromptReco
        (isReReco, isPromptReco) = (False, False)
        if getattr(workload.data.request.schema, "RequestType", None) == 'ReReco':
            isReReco = True
        # Yes, few people like magic strings, but have a look at :
        # https://github.com/dmwm/T0/blob/master/src/python/T0/RunConfig/RunConfigAPI.py#L718
        # Might be safe enough
        # FIXME: in TaskArchiver, add a test to make sure that the dataset makes sense (procDataset ~= /a/ERA-PromptReco-vVERSON/DQM)
        if re.search('PromptReco', workload.name()):
            isPromptReco = True
        if not (isReReco or isPromptReco):
            return

        self.assertTrue(isReReco)
        self.assertFalse(isPromptReco)

        # We are not interested if it's not a PromptReco or a ReReco
        if not (isReReco or isPromptReco):
            return
        if isReReco:
            release = getattr(workload.data.request.schema, "CMSSWVersion")
            if not release:
                logging.info("no release for %s, bailing out", workload.name())
        else:
            release = getattr(workload.tasks.Reco.steps.cmsRun1.application.setup, "cmsswVersion")
            if not release:
                logging.info("no release for %s, bailing out", workload.name())

        self.assertEqual(release, "test_compops_CMSSW_5_3_6_patch1")
        # If all is true, get the run numbers processed by this worklfow
        runList = listRunsWorkflow.execute(workflow=workload.name())
        self.assertEqual([207214, 207215], runList)
        # GO to DQM GUI, get what you want
        # https://cmsweb.cern.ch/dqm/offline/jsonfairy/archive/211313/PAMuon/HIRun2013-PromptReco-v1/DQM/DQM/TimerService/event
        for dataset in interestingDatasets:
            (nothing, PD, procDataSet, dataTier) = dataset.split('/')
            worthPoints = {}
            for run in runList:
                responseJSON = self.getPerformanceFromDQM(dqmUrl, dataset, run)
                worthPoints.update(
                    self.filterInterestingPerfPoints(responseJSON, perfDashBoardMinLumi, perfDashBoardMaxLumi))

            # Publish dataset performance to DashBoard.
            if not self.publishPerformanceDashBoard(dashBoardUrl, PD, release, worthPoints):
                logging.info("something went wrong when publishing dataset %s to DashBoard", dataset)

        return


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
ReproducibleSeeding_t.py

Created by Dave Evans on 2010-08-30.
Copyright (c) 2010 Fermilab. All rights reserved.
"""

import sys
import os
import unittest

from WMCore.JobSplitting.Generators.ReproducibleSeeding import ReproducibleSeeding
from WMQuality.TestInitCouchApp import TestInitCouchApp
from WMCore.DataStructs.Job import Job
from WMCore.Database.CMSCouch import Database, Document



class ReproducibleSeedingTests(unittest.TestCase):
    def setUp(self):
        self.testInit = TestInitCouchApp("ReproducibleSeedingTest")
        self.testInit.setupCouch("seeding_config_cache", "GroupUser", "ConfigCache")
        self.database = Database(self.testInit.couchDbName, self.testInit.couchUrl)

        self.documentId = None

    def tearDown(self):
        self.testInit.tearDownCouch()
        return


    def testA(self):
        """instantiate"""

        document = Document()
        document[u'pset_tweak_details'] = {}
        document[u'pset_tweak_details'][u'process'] = {}
        document[u'pset_tweak_details'][u'process'][u'RandomNumberGeneratorService'] = {}
        document[u'pset_tweak_details'][u'process'][u'RandomNumberGeneratorService'][u'seed1'] = {}
        document[u'pset_tweak_details'][u'process'][u'RandomNumberGeneratorService'][u'seed2'] = {}
        document[u'pset_tweak_details'][u'process'][u'RandomNumberGeneratorService'][u'seed3'] = {}

        document = self.database.commitOne(document)[0]
        seeder = ReproducibleSeeding(CouchUrl = self.testInit.couchUrl,
                                     CouchDBName = self.testInit.couchDbName,
                                     ConfigCacheDoc = document[u'id'])

        job = Job("testjob")
        seeder(job)
        baggage = job.getBaggage()
        seed1 = getattr(baggage.process.RandomNumberGeneratorService, "seed1", None)
        self.assertTrue(seed1 != None)


if __name__ == '__main__':
    unittest.main()
